@@ -1,7 +1,6 @@
 /* Evolution calendar - Data model for ETable
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2000 Ximian, Inc.
+ * Copyright (C) 2004 Ximian, Inc.
  *
  * Authors: Rodrigo Moya <rodrigo@ximian.com>
  *
@@ -19,11 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <math.h>
+#include <string.h>
 #include <gtk/gtkmessagedialog.h>
 #include <libgnome/gnome-i18n.h>
-#include <gal/util/e-util.h>
 #include "calendar-config.h"
 #include "e-cal-model-tasks.h"
 #include "e-cell-date-edit-text.h"
@@ -32,9 +34,7 @@
 struct _ECalModelTasksPrivate {
 };
 
-static void ecmt_class_init (ECalModelTasksClass *klass);
-static void ecmt_init (ECalModelTasks *model, ECalModelTasksClass *klass);
-static void ecmt_finalize (GObject *object);
+static void e_cal_model_tasks_finalize (GObject *object);
 
 static int ecmt_column_count (ETableModel *etm);
 static void *ecmt_value_at (ETableModel *etm, int col, int row);
@@ -50,21 +50,16 @@ static const char *ecmt_get_color_for_component (ECalModel *model, ECalModelComp
 static void ecmt_fill_component_from_model (ECalModel *model, ECalModelComponent *comp_data,
 					    ETableModel *source_model, gint row);
 
-static GObjectClass *parent_class = NULL;
-
-E_MAKE_TYPE (e_cal_model_tasks, "ECalModelTasks", ECalModelTasks, ecmt_class_init,
-	     ecmt_init, E_TYPE_CAL_MODEL);
+G_DEFINE_TYPE (ECalModelTasks, e_cal_model_tasks, E_TYPE_CAL_MODEL);
 
 static void
-ecmt_class_init (ECalModelTasksClass *klass)
+e_cal_model_tasks_class_init (ECalModelTasksClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	ETableModelClass *etm_class = E_TABLE_MODEL_CLASS (klass);
 	ECalModelClass *model_class = E_CAL_MODEL_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
-	object_class->finalize = ecmt_finalize;
+	object_class->finalize = e_cal_model_tasks_finalize;
 
 	etm_class->column_count = ecmt_column_count;
 	etm_class->value_at = ecmt_value_at;
@@ -81,7 +76,7 @@ ecmt_class_init (ECalModelTasksClass *klass)
 }
 
 static void
-ecmt_init (ECalModelTasks *model, ECalModelTasksClass *klass)
+e_cal_model_tasks_init (ECalModelTasks *model)
 {
 	ECalModelTasksPrivate *priv;
 
@@ -92,7 +87,7 @@ ecmt_init (ECalModelTasks *model, ECalModelTasksClass *klass)
 }
 
 static void
-ecmt_finalize (GObject *object)
+e_cal_model_tasks_finalize (GObject *object)
 {
 	ECalModelTasksPrivate *priv;
 	ECalModelTasks *model = (ECalModelTasks *) object;
@@ -105,8 +100,8 @@ ecmt_finalize (GObject *object)
 		model->priv = NULL;
 	}
 
-	if (parent_class->finalize)
-		parent_class->finalize (object);
+	if (G_OBJECT_CLASS (e_cal_model_tasks_parent_class)->finalize)
+		G_OBJECT_CLASS (e_cal_model_tasks_parent_class)->finalize (object);
 }
 
 /* ETableModel methods */
@@ -170,6 +165,33 @@ ensure_task_complete (ECalModelComponent *comp_data, time_t completed_date)
 		icalcomponent_add_property (comp_data->icalcomp, icalproperty_new_status (ICAL_STATUS_COMPLETED));
 }
 
+static void
+ensure_task_partially_complete (ECalModelComponent *comp_data)
+{
+	icalproperty *prop;
+
+	/* Date Completed. */
+	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_COMPLETED_PROPERTY);
+	if (prop) {
+		icalcomponent_remove_property (comp_data->icalcomp, prop);
+		icalproperty_free (prop);
+	}
+
+	/* Percent. */
+	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_PERCENTCOMPLETE_PROPERTY);
+	if (!prop)
+		icalcomponent_add_property (comp_data->icalcomp, icalproperty_new_percentcomplete (50));
+	else if (icalproperty_get_percentcomplete (prop) == 0 || icalproperty_get_percentcomplete (prop) == 100)
+		icalproperty_set_percentcomplete (prop, 50);
+
+	/* Status. */
+	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_STATUS_PROPERTY);
+	if (prop)
+		icalproperty_set_status (prop, ICAL_STATUS_INPROCESS);
+	else
+		icalcomponent_add_property (comp_data->icalcomp, icalproperty_new_status (ICAL_STATUS_INPROCESS));
+}
+
 
 /* This makes sure a task is marked as incomplete. It clears the
    "Date Completed" property. If the percent is set to 100 it removes it,
@@ -214,7 +236,7 @@ get_completed (ECalModelComponent *comp_data)
 			return NULL;
 
 		tt_completed = icalproperty_get_completed (prop);
-		if (!icaltime_is_valid_time (tt_completed))
+		if (!icaltime_is_valid_time (tt_completed) || icaltime_is_null_time (tt_completed))
 			return NULL;
 
 		comp_data->completed = g_new0 (ECellDateEditValue, 1);
@@ -244,7 +266,7 @@ get_due (ECalModelComponent *comp_data)
 			return NULL;
 
 		tt_due = icalproperty_get_due (prop);
-		if (!icaltime_is_valid_time (tt_due))
+		if (!icaltime_is_valid_time (tt_due) || icaltime_is_null_time (tt_due))
 			return NULL;
 
 		comp_data->due = g_new0 (ECellDateEditValue, 1);
@@ -444,7 +466,7 @@ ecmt_value_at (ETableModel *etm, int col, int row)
 	g_return_val_if_fail (row >= 0 && row < e_table_model_row_count (etm), NULL);
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->value_at (etm, col, row);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->value_at (etm, col, row);
 
 	comp_data = e_cal_model_get_component_at (E_CAL_MODEL (model), row);
 	if (!comp_data)
@@ -513,23 +535,52 @@ set_complete (ECalModelComponent *comp_data, const void *value)
 static void
 set_due (ECalModelComponent *comp_data, const void *value)
 {
-	icalproperty *prop;
 	ECellDateEditValue *dv = (ECellDateEditValue *) value;
-
+	icalproperty *prop;
+	icalparameter *param;
+	const char *tzid;
+	
 	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_DUE_PROPERTY);
+	if (prop)
+		param = icalproperty_get_first_parameter (prop, ICAL_TZID_PARAMETER);
+	else
+		param = NULL;
 
+	/* If we are setting the property to NULL (i.e. removing it), then
+	   we remove it if it exists. */
 	if (!dv) {
 		if (prop) {
 			icalcomponent_remove_property (comp_data->icalcomp, prop);
 			icalproperty_free (prop);
 		}
+
+		return;
+	}
+	
+	/* If the TZID is set to "UTC", we set the is_utc flag. */
+	tzid = dv->zone ? icaltimezone_get_tzid (dv->zone) : "UTC";
+	if (tzid && !strcmp (tzid, "UTC"))
+		dv->tt.is_utc = 1;
+	else
+		dv->tt.is_utc = 0;
+
+	if (prop) {
+		icalproperty_set_due (prop, dv->tt);
 	} else {
-		if (prop)
-			icalproperty_set_due (prop, dv->tt);
-		else {
-			prop = icalproperty_new_due (dv->tt);
-			icalcomponent_add_property (comp_data->icalcomp, prop);
+		prop = icalproperty_new_due (dv->tt);
+		icalcomponent_add_property (comp_data->icalcomp, prop);
+	}
+
+	/* If the TZID is set to "UTC", we don't want to save the TZID. */
+	if (tzid && strcmp (tzid, "UTC")) {
+		if (param) {
+			icalparameter_set_tzid (param, (char *) tzid);
+		} else {
+			param = icalparameter_new_tzid ((char *) tzid);
+			icalproperty_add_parameter (prop, param);
 		}
+	} else if (param) {
+		icalproperty_remove_parameter (prop, ICAL_TZID_PARAMETER);
 	}
 }
 
@@ -610,17 +661,25 @@ set_status (ECalModelComponent *comp_data, const char *value)
 		icalcomponent_add_property (comp_data->icalcomp, prop);
 	}
 
-/* 	if (status == ICAL_STATUS_NEEDSACTION) { */
-/* 		percent = 0; */
-/* 		e_cal_component_set_percent (comp, &percent); */
-/* 		e_cal_component_set_completed (comp, NULL); */
-/* 	} else if (status == ICAL_STATUS_INPROCESS) {	 */
-/* 		ensure_task_not_complete (comp);	 */
-/* 		percent = 50; */
-/* 		e_cal_component_set_percent (comp, &percent); */
-/* 	} else if (status == ICAL_STATUS_COMPLETED) { */
-/* 		ensure_task_complete (comp, -1); */
-/* 	} */
+	switch (status) {
+	case ICAL_STATUS_NEEDSACTION:
+		ensure_task_not_complete (comp_data);
+		break;
+
+	case ICAL_STATUS_INPROCESS:
+		ensure_task_partially_complete (comp_data);
+		break;
+
+	case ICAL_STATUS_CANCELLED:
+		ensure_task_not_complete (comp_data);
+		break;
+
+	case ICAL_STATUS_COMPLETED:
+		ensure_task_complete (comp_data, -1);
+		break;		
+	default:
+		break;
+	}
 }
 
 static void
@@ -652,7 +711,12 @@ set_percent (ECalModelComponent *comp_data, const void *value)
 		if (percent == 100)
 			ensure_task_complete (comp_data, -1);
 		else {
-			ensure_task_not_complete (comp_data);
+			prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_COMPLETED_PROPERTY);
+			if (prop) {
+				icalcomponent_remove_property (comp_data->icalcomp, prop);
+				icalproperty_free (prop);
+			}
+			
 			if (percent > 0)
 				set_status (comp_data, _("In Progress"));
 		}
@@ -710,9 +774,6 @@ ecmt_set_value_at (ETableModel *etm, int col, int row, const void *value)
 	ECalModelTasksPrivate *priv;
 	ECalModelComponent *comp_data;
 	ECalModelTasks *model = (ECalModelTasks *) etm;
-	icaltimetype start_tt, due_tt;
-	ECellDateEditValue *dv;
-	GtkWidget *dialog;
 
 	g_return_if_fail (E_IS_CAL_MODEL_TASKS (model));
 
@@ -721,32 +782,15 @@ ecmt_set_value_at (ETableModel *etm, int col, int row, const void *value)
 	g_return_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST);
 	g_return_if_fail (row >= 0 && row < e_table_model_row_count (etm));
 
+	if (col < E_CAL_MODEL_FIELD_LAST) {
+		E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->set_value_at (etm, col, row, value);
+		return;
+	}
+
 	comp_data = e_cal_model_get_component_at (E_CAL_MODEL (model), row);
 	if (!comp_data)
 		return;
 
-	if (col < E_CAL_MODEL_FIELD_LAST) {
-		if (col == E_CAL_MODEL_FIELD_DTSTART) {
-			dv = (ECellDateEditValue *) value;
-			start_tt = dv->tt;
-			due_tt = icalcomponent_get_due (comp_data->icalcomp);
-
-			if (icaltime_compare (start_tt, due_tt) > 0) {
-				dialog = gtk_message_dialog_new (NULL, 0,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Due date is before start date!"));
-				g_signal_connect (dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
-				gtk_widget_show (dialog);
-
-				return;
-			}
-		}
-
-		E_TABLE_MODEL_CLASS (parent_class)->set_value_at (etm, col, row, value);
-		return;
-	}
-
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_COMPLETED :
 		set_completed (model, comp_data, value);
@@ -755,20 +799,6 @@ ecmt_set_value_at (ETableModel *etm, int col, int row, const void *value)
 		set_complete (comp_data, value);
 		break;
 	case E_CAL_MODEL_TASKS_FIELD_DUE :
-		dv = (ECellDateEditValue *) value;
-		start_tt = icalcomponent_get_dtstart (comp_data->icalcomp);
-		due_tt = dv->tt;
-		
-		if (icaltime_compare (start_tt, due_tt) > 0) {
-			dialog = gtk_message_dialog_new (NULL, 0,
-					GTK_MESSAGE_ERROR,
-					GTK_BUTTONS_OK,
-					_("Due date is before start date!"));
-			g_signal_connect (dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
-			gtk_widget_show (dialog);
-			
-			return;
-		}
 		set_due (comp_data, value);
 		break;
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
@@ -780,6 +810,9 @@ ecmt_set_value_at (ETableModel *etm, int col, int row, const void *value)
 	case E_CAL_MODEL_TASKS_FIELD_PRIORITY :
 		set_priority (comp_data, value);
 		break;
+	case E_CAL_MODEL_TASKS_FIELD_STATUS :
+		set_status (comp_data, value);
+		break;
 	case E_CAL_MODEL_TASKS_FIELD_URL :
 		set_url (comp_data, value);
 		break;
@@ -804,12 +837,10 @@ ecmt_is_cell_editable (ETableModel *etm, int col, int row)
 	priv = model->priv;
 
 	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST, FALSE);
-
-	/* FIXME: We can't check this as 'click-to-add' passes row 0. */
-	/* g_return_val_if_fail (row >= 0 && row < e_table_model_get_row_count (etm), FALSE); */
+	g_return_val_if_fail (row >= -1 || (row >= 0 && row < e_table_model_row_count (etm)), FALSE);
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->is_cell_editable (etm, col, row);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->is_cell_editable (etm, col, row);
 
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_COMPLETED :
@@ -818,6 +849,7 @@ ecmt_is_cell_editable (ETableModel *etm, int col, int row)
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
 	case E_CAL_MODEL_TASKS_FIELD_PERCENT :
 	case E_CAL_MODEL_TASKS_FIELD_PRIORITY :
+	case E_CAL_MODEL_TASKS_FIELD_STATUS :
 	case E_CAL_MODEL_TASKS_FIELD_URL :
 		return TRUE;
 	}
@@ -831,7 +863,7 @@ ecmt_duplicate_value (ETableModel *etm, int col, const void *value)
 	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST, NULL);
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->duplicate_value (etm, col, value);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->duplicate_value (etm, col, value);
 
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
@@ -841,8 +873,17 @@ ecmt_duplicate_value (ETableModel *etm, int col, const void *value)
 		return g_strdup (value);
 	case E_CAL_MODEL_TASKS_FIELD_COMPLETED :
 	case E_CAL_MODEL_TASKS_FIELD_DUE :
-		/* FIXME */
+		if (value) {
+			ECellDateEditValue *dv, *orig_dv;
+
+			orig_dv = (ECellDateEditValue *) value;
+			dv = g_new0 (ECellDateEditValue, 1);
+			*dv = *orig_dv;
+
+			return dv;
+		}
 		break;
+
 	case E_CAL_MODEL_TASKS_FIELD_COMPLETE :
 	case E_CAL_MODEL_TASKS_FIELD_PERCENT :
 	case E_CAL_MODEL_TASKS_FIELD_OVERDUE :
@@ -858,7 +899,7 @@ ecmt_free_value (ETableModel *etm, int col, void *value)
 	g_return_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST);
 	
 	if (col < E_CAL_MODEL_FIELD_LAST) {
-		E_TABLE_MODEL_CLASS (parent_class)->free_value (etm, col, value);
+		E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->free_value (etm, col, value);
 		return;
 	}
 
@@ -888,7 +929,7 @@ ecmt_initialize_value (ETableModel *etm, int col)
 	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST, NULL);
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->initialize_value (etm, col);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->initialize_value (etm, col);
 
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
@@ -920,7 +961,7 @@ ecmt_value_is_empty (ETableModel *etm, int col, const void *value)
 	priv = model->priv;
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->value_is_empty (etm, col, value);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->value_is_empty (etm, col, value);
 
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
@@ -946,11 +987,11 @@ ecmt_value_to_string (ETableModel *etm, int col, const void *value)
 {
 	ECalModelTasks *model = (ECalModelTasks *) etm;
 
-	g_return_val_if_fail (E_IS_CAL_MODEL_TASKS (model), NULL);
-	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST, NULL);
+	g_return_val_if_fail (E_IS_CAL_MODEL_TASKS (model), g_strdup (""));
+	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_TASKS_FIELD_LAST, g_strdup (""));
 
 	if (col < E_CAL_MODEL_FIELD_LAST)
-		return E_TABLE_MODEL_CLASS (parent_class)->value_to_string (etm, col, value);
+		return E_TABLE_MODEL_CLASS (e_cal_model_tasks_parent_class)->value_to_string (etm, col, value);
 
 	switch (col) {
 	case E_CAL_MODEL_TASKS_FIELD_GEO :
@@ -963,7 +1004,7 @@ ecmt_value_to_string (ETableModel *etm, int col, const void *value)
 		return e_cal_model_date_value_to_string (E_CAL_MODEL (model), value);
 	case E_CAL_MODEL_TASKS_FIELD_COMPLETE :
 	case E_CAL_MODEL_TASKS_FIELD_OVERDUE :
-		return value ? _("Yes") : _("No");
+		return g_strdup (value ? _("Yes") : _("No"));
 	case E_CAL_MODEL_TASKS_FIELD_PERCENT :
 		if (GPOINTER_TO_INT (value) < 0)
 			return g_strdup ("N/A");
@@ -971,7 +1012,7 @@ ecmt_value_to_string (ETableModel *etm, int col, const void *value)
 			return g_strdup_printf ("%i%%", GPOINTER_TO_INT (value));
 	}
 
-	return NULL;
+	return g_strdup ("");
 }
 
 /* ECalModel class methods */
@@ -993,29 +1034,37 @@ ecmt_get_color_for_component (ECalModel *model, ECalModelComponent *comp_data)
 		break;
 	}
 
-	return E_CAL_MODEL_CLASS (parent_class)->get_color_for_component (model, comp_data);
+	return E_CAL_MODEL_CLASS (e_cal_model_tasks_parent_class)->get_color_for_component (model, comp_data);
 }
 
 static void
 ecmt_fill_component_from_model (ECalModel *model, ECalModelComponent *comp_data,
 				ETableModel *source_model, gint row)
 {
+	void *value;
+	
 	g_return_if_fail (E_IS_CAL_MODEL_TASKS (model));
 	g_return_if_fail (comp_data != NULL);
 	g_return_if_fail (E_IS_TABLE_MODEL (source_model));
 
-	set_completed ((ECalModelTasks *) model, comp_data,
-		       e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_COMPLETED, row));
+	/* This just makes sure if anything indicates completion, all
+	 * three fields do or if percent is 0, status is sane */
+
+	value = e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_COMPLETED, row);
+	set_completed ((ECalModelTasks *) model, comp_data, value);
+	if (!value) {
+		value = e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_PERCENT, row);
+		set_percent (comp_data, value);
+		if (GPOINTER_TO_INT (value) != 100 && GPOINTER_TO_INT (value) != 0)
+			set_status (comp_data, e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_STATUS, row));
+	}
+	
 	set_due (comp_data,
 		 e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_DUE, row));
 	set_geo (comp_data,
 		 e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_GEO, row));
-	set_percent (comp_data,
-		     e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_PERCENT, row));
 	set_priority (comp_data,
 		      e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_PRIORITY, row));
-	set_status (comp_data,
-		    e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_STATUS, row));
 	set_url (comp_data,
 		 e_table_model_value_at (source_model, E_CAL_MODEL_TASKS_FIELD_URL, row));
 }
@@ -1044,6 +1093,11 @@ e_cal_model_tasks_mark_task_complete (ECalModelTasks *model, gint model_row)
 	priv = model->priv;
 
 	comp_data = e_cal_model_get_component_at (E_CAL_MODEL (model), model_row);
-	if (comp_data)
+	if (comp_data) {
+		e_table_model_pre_change (E_TABLE_MODEL (model));
+
 		ensure_task_complete (comp_data, -1);
+
+		e_table_model_row_changed (E_TABLE_MODEL (model), model_row);
+	}
 }
