@@ -43,6 +43,7 @@
 #include <gtkhtml/gtkhtml-embedded.h>
 #include <gtkhtml/gtkhtml-stream.h>
 #include <libedataserver/e-source-list.h>
+#include <libedataserverui/e-source-option-menu.h>
 #include <libical/ical.h>
 #include <libecal/e-cal-component.h>
 #include <libecal/e-cal-time-util.h>
@@ -50,7 +51,6 @@
 #include <e-util/e-time-utils.h>
 #include <e-util/e-dialog-widgets.h>
 #include <e-util/e-html-utils.h>
-#include <widgets/misc/e-source-option-menu.h>
 #include "dialogs/delete-error.h"
 #include "calendar-config.h"
 #include "itip-utils.h"
@@ -104,22 +104,17 @@ struct _EItipControlPrivate {
 #define HTML_BODY_END   "</body>"
 #define HTML_FOOTER     "</html>"
 
-static void class_init	(EItipControlClass	 *klass);
-static void init	(EItipControl		 *itip);
-static void destroy	(GtkObject               *obj);
+static void e_itip_control_destroy	(GtkObject               *obj);
 
 static void find_my_address (EItipControl *itip, icalcomponent *ical_comp, icalparameter_partstat *status);
 static void url_requested_cb (GtkHTML *html, const gchar *url, GtkHTMLStream *handle, gpointer data);
 static gboolean object_requested_cb (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data);
 static void ok_clicked_cb (GtkWidget *widget, gpointer data);
 
-static GtkVBoxClass *parent_class = NULL;
-
-E_MAKE_TYPE (e_itip_control, "EItipControl", EItipControl, class_init, init,
-	     GTK_TYPE_VBOX);
+G_DEFINE_TYPE (EItipControl, e_itip_control, GTK_TYPE_VBOX);
 
 static void
-class_init (EItipControlClass *klass)
+e_itip_control_class_init (EItipControlClass *klass)
 {
 	GObjectClass *object_class;
 	GtkObjectClass *gtkobject_class;
@@ -127,9 +122,7 @@ class_init (EItipControlClass *klass)
 	object_class = G_OBJECT_CLASS (klass);
 	gtkobject_class = GTK_OBJECT_CLASS (klass);
 	
-	parent_class = g_type_class_peek_parent (klass);
-
-	gtkobject_class->destroy = destroy;
+	gtkobject_class->destroy = e_itip_control_destroy;
 }
 
 static void
@@ -389,7 +382,7 @@ html_destroyed (gpointer data)
 }
 
 static void
-init (EItipControl *itip)
+e_itip_control_init (EItipControl *itip)
 {
 	EItipControlPrivate *priv;
 	GtkWidget *scrolled_window;
@@ -489,7 +482,7 @@ clean_up (EItipControl *itip)
 }
 
 static void
-destroy (GtkObject *obj)
+e_itip_control_destroy (GtkObject *obj)
 {
 	EItipControl *itip = E_ITIP_CONTROL (obj);
 	EItipControlPrivate *priv;
@@ -519,7 +512,7 @@ destroy (GtkObject *obj)
 		itip->priv = NULL;
 	}
 	
-	(* GTK_OBJECT_CLASS (parent_class)->destroy) (obj);
+	(* GTK_OBJECT_CLASS (e_itip_control_parent_class)->destroy) (obj);
 }
 
 GtkWidget *
