@@ -103,8 +103,6 @@ enum {
 
 static gint mts_signals [LAST_SIGNAL] = { 0 };
 
-static void e_meeting_time_selector_class_init (EMeetingTimeSelectorClass * klass);
-static void e_meeting_time_selector_init (EMeetingTimeSelector * mts);
 static void e_meeting_time_selector_destroy (GtkObject *object);
 static void e_meeting_time_selector_alloc_named_color (EMeetingTimeSelector * mts,
 						       const char *name, GdkColor *c);
@@ -209,11 +207,7 @@ static void row_inserted_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter
 static void row_changed_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static void row_deleted_cb (GtkTreeModel *model, GtkTreePath *path, gpointer data);
 
-static GtkTableClass *parent_class;
-
-E_MAKE_TYPE (e_meeting_time_selector, "EMeetingTimeSelector", EMeetingTimeSelector,
-	     e_meeting_time_selector_class_init, e_meeting_time_selector_init,
-	     GTK_TYPE_TABLE);
+G_DEFINE_TYPE (EMeetingTimeSelector, e_meeting_time_selector, GTK_TYPE_TABLE);
 
 static void
 e_meeting_time_selector_class_init (EMeetingTimeSelectorClass * klass)
@@ -221,7 +215,6 @@ e_meeting_time_selector_class_init (EMeetingTimeSelectorClass * klass)
 	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = g_type_class_peek_parent (klass);
 	object_class = (GtkObjectClass *) klass;
 	widget_class = (GtkWidgetClass *) klass;
 
@@ -277,6 +270,7 @@ e_meeting_time_selector_construct (EMeetingTimeSelector * mts, EMeetingStore *em
 	guchar stipple_bits[] = {
 		0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80,
 	};
+	AtkObject *a11y_label, *a11y_date_edit;
 
 	/* The default meeting time is the nearest half-hour interval in the
 	   future, in working hours. */
@@ -591,6 +585,14 @@ e_meeting_time_selector_construct (EMeetingTimeSelector * mts, EMeetingStore *em
 	gtk_widget_show (table);
 
 	mts->start_date_edit = e_date_edit_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), mts->start_date_edit);
+	a11y_label = gtk_widget_get_accessible (label);
+	a11y_date_edit = gtk_widget_get_accessible (mts->start_date_edit);
+	if (a11y_label != NULL && a11y_date_edit != NULL) {
+		atk_object_add_relationship (a11y_date_edit,
+					ATK_RELATION_LABELLED_BY,
+					a11y_label);
+	}
 	e_date_edit_set_show_time (E_DATE_EDIT (mts->start_date_edit), TRUE);
 	e_date_edit_set_use_24_hour_format (E_DATE_EDIT (mts->start_date_edit),
 					    calendar_config_get_24_hour_format ());
@@ -610,6 +612,14 @@ e_meeting_time_selector_construct (EMeetingTimeSelector * mts, EMeetingStore *em
 	gtk_widget_show (label);
 
 	mts->end_date_edit = e_date_edit_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), mts->end_date_edit);
+	a11y_label = gtk_widget_get_accessible (label);
+	a11y_date_edit = gtk_widget_get_accessible (mts->end_date_edit);
+	if (a11y_label != NULL && a11y_date_edit != NULL) {
+		atk_object_add_relationship (a11y_date_edit,
+					ATK_RELATION_LABELLED_BY,
+					a11y_label);
+	}
 	e_date_edit_set_show_time (E_DATE_EDIT (mts->end_date_edit), TRUE);
 	e_date_edit_set_use_24_hour_format (E_DATE_EDIT (mts->end_date_edit),
 					    calendar_config_get_24_hour_format ());
@@ -816,8 +826,8 @@ e_meeting_time_selector_destroy (GtkObject *object)
 	mts->display_top = NULL;
 	mts->display_main = NULL;
 	
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(*GTK_OBJECT_CLASS (parent_class)->destroy)(object);
+	if (GTK_OBJECT_CLASS (e_meeting_time_selector_parent_class)->destroy)
+		(*GTK_OBJECT_CLASS (e_meeting_time_selector_parent_class)->destroy)(object);
 }
 
 
@@ -826,8 +836,8 @@ e_meeting_time_selector_realize (GtkWidget *widget)
 {
 	EMeetingTimeSelector *mts;
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize)
-		(*GTK_WIDGET_CLASS (parent_class)->realize)(widget);
+	if (GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->realize)
+		(*GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->realize)(widget);
 
 	mts = E_MEETING_TIME_SELECTOR (widget);
 
@@ -845,8 +855,8 @@ e_meeting_time_selector_unrealize (GtkWidget *widget)
 	gdk_gc_unref (mts->color_key_gc);
 	mts->color_key_gc = NULL;
 
-	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-		(*GTK_WIDGET_CLASS (parent_class)->unrealize)(widget);
+	if (GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->unrealize)
+		(*GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->unrealize)(widget);
 }
 
 static int
@@ -878,8 +888,8 @@ e_meeting_time_selector_style_set (GtkWidget *widget,
 	GtkTreePath *path;
 	GdkRectangle cell_area;
 
-	if (GTK_WIDGET_CLASS (parent_class)->style_set)
-		(*GTK_WIDGET_CLASS (parent_class)->style_set)(widget, previous_style);
+	if (GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->style_set)
+		(*GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->style_set)(widget, previous_style);
 
 	mts = E_MEETING_TIME_SELECTOR (widget);
 
@@ -943,8 +953,8 @@ e_meeting_time_selector_expose_event (GtkWidget *widget,
 
 	e_meeting_time_selector_draw_shadow (mts);
 
-	if (GTK_WIDGET_CLASS (parent_class)->expose_event)
-		(*GTK_WIDGET_CLASS (parent_class)->expose_event)(widget, event);
+	if (GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->expose_event)
+		(*GTK_WIDGET_CLASS (e_meeting_time_selector_parent_class)->expose_event)(widget, event);
 
 	return FALSE;
 }
