@@ -20,7 +20,18 @@
  * Author: Rodrigo Moya <rodrigo@ximian.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <dirent.h>
+#include <fcntl.h>
+#include <errno.h>
+
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
@@ -29,10 +40,12 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkprogressbar.h>
-#include <gal/util/e-util.h>
 #include <libecal/e-cal.h>
 #include <e-util/e-bconf-map.h>
 #include <e-util/e-folder-map.h>
+#include <libedataserver/e-dbhash.h>
+#include <libedataserver/e-xml-hash-utils.h>
+#include "calendar-config.h"
 #include "calendar-config-keys.h"
 #include "migration.h"
 
@@ -351,6 +364,7 @@ migrate_ical_folder (char *old_path, ESourceGroup *dest_group, char *source_name
 
 #define WEBCAL_BASE_URI "webcal://"
 #define CONTACTS_BASE_URI "contacts://"
+#define BAD_CONTACTS_BASE_URI "contact://"
 #define PERSONAL_RELATIVE_URI "system"
 
 static ESourceGroup *
@@ -367,6 +381,7 @@ create_calendar_contact_source (ESourceList *source_list)
 	e_source_group_add_source (group, source, -1);
 	g_object_unref (source);
 
+	e_source_set_color (source, 0xFED4D3);
 	e_source_group_set_readonly (group, TRUE);
 
 	return group;
@@ -404,6 +419,12 @@ create_calendar_sources (CalendarComponent *component,
 
 			group = E_SOURCE_GROUP (g->data);
 
+			if (!strcmp (BAD_CONTACTS_BASE_URI, e_source_group_peek_base_uri (group)))
+				e_source_group_set_base_uri (group, CONTACTS_BASE_URI);
+
+			if (!strcmp (base_uri, e_source_group_peek_base_uri (group)))
+				e_source_group_set_base_uri (group, base_uri_proto);
+
 			if (!*on_this_computer && !strcmp (base_uri_proto, e_source_group_peek_base_uri (group)))
 				*on_this_computer = g_object_ref (group);
 			else if (!*on_the_web && !strcmp (WEBCAL_BASE_URI, e_source_group_peek_base_uri (group)))
@@ -437,7 +458,18 @@ create_calendar_sources (CalendarComponent *component,
 		/* Create the default Person calendar */
 		ESource *source = e_source_new (_("Personal"), PERSONAL_RELATIVE_URI);
 		e_source_group_add_source (*on_this_computer, source, -1);
+		
+		if (!calendar_config_get_primary_calendar () && !calendar_config_get_calendars_selected ()) {
+			GSList selected;
 
+			calendar_config_set_primary_calendar (e_source_peek_uid (source));
+
+			selected.data = (gpointer)e_source_peek_uid (source);
+			selected.next = NULL;
+			calendar_config_set_calendars_selected (&selected);
+		}
+		
+		e_source_set_color (source, 0xBECEDD);
 		*personal_source = source;
 	}
 
@@ -475,8 +507,7 @@ create_task_sources (TasksComponent *component,
 	*personal_source = NULL;
 	
 	base_uri = g_build_filename (tasks_component_peek_base_directory (component),
-				     "/tasks/local/",
-				     NULL);
+				     "tasks", "local", NULL);
 
 	base_uri_proto = g_strconcat ("file://", base_uri, NULL);
 
@@ -521,6 +552,17 @@ create_task_sources (TasksComponent *component,
 		ESource *source = e_source_new (_("Personal"), PERSONAL_RELATIVE_URI);
 		e_source_group_add_source (*on_this_computer, source, -1);
 
+		if (!calendar_config_get_primary_tasks () && !calendar_config_get_tasks_selected ()) {
+			GSList selected;
+
+			calendar_config_set_primary_tasks (e_source_peek_uid (source));
+
+			selected.data = (gpointer)e_source_peek_uid (source);
+			selected.next = NULL;
+			calendar_config_set_tasks_selected (&selected);
+		}
+
+		e_source_set_color (source, 0xBECEDD);
 		*personal_source = source;
 	}
 
@@ -536,12 +578,129 @@ create_task_sources (TasksComponent *component,
 	g_free (base_uri);
 }
 
+static void
+migrate_pilot_db_key (const char *key, gpointer user_data)
+{
+	EXmlHash *xmlhash = user_data;
+	
+	e_xmlhash_add (xmlhash, key, "");
+}
+
+static void
+migrate_pilot_data (const char *component, const char *conduit, const char *old_path, const char *new_path)
+{
+	char *changelog, *map;
+	struct dirent *dent;
+	const char *ext;
+	char *filename;
+	DIR *dir;
+	
+	if (!(dir = opendir (old_path)))
+		return;
+	
+	map = g_alloca (12 + strlen (conduit));
+	sprintf (map, "pilot-map-%s-", conduit);
+	
+	changelog = g_alloca (24 + strlen (conduit));
+	sprintf (changelog, "pilot-sync-evolution-%s-", conduit);
+	
+	while ((dent = readdir (dir))) {
+		if (!strncmp (dent->d_name, map, strlen (map)) &&
+		    ((ext = strrchr (dent->d_name, '.')) && !strcmp (ext, ".xml"))) {
+			/* pilot map file - src and dest file formats are identical */
+			unsigned char inbuf[4096];
+			size_t nread, nwritten;
+			int fd0, fd1;
+			ssize_t n;
+			
+			filename = g_build_filename (old_path, dent->d_name, NULL);
+			if ((fd0 = open (filename, O_RDONLY)) == -1) {
+				g_free (filename);
+				continue;
+			}
+			
+			g_free (filename);
+			filename = g_build_filename (new_path, dent->d_name, NULL);
+			if ((fd1 = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0666)) == -1) {
+				g_free (filename);
+				close (fd0);
+				continue;
+			}
+			
+			do {
+				do {
+					n = read (fd0, inbuf, sizeof (inbuf));
+				} while (n == -1 && errno == EINTR);
+				
+				if (n < 1)
+					break;
+				
+				nread = n;
+				nwritten = 0;
+				do {
+					do {
+						n = write (fd1, inbuf + nwritten, nread - nwritten);
+					} while (n == -1 && errno == EINTR);
+					
+					if (n > 0)
+						nwritten += n;
+				} while (nwritten < nread && n != -1);
+				
+				if (n == -1)
+					break;
+			} while (1);
+			
+			if (n != -1)
+				n = fsync (fd1);
+			
+			if (n == -1) {
+				g_warning ("Failed to migrate %s: %s", dent->d_name, strerror (errno));
+				unlink (filename);
+			}
+			
+			close (fd0);
+			close (fd1);
+			g_free (filename);
+		} else if (!strncmp (dent->d_name, changelog, strlen (changelog)) &&
+			   ((ext = strrchr (dent->d_name, '.')) && !strcmp (ext, ".db"))) {
+			/* src and dest formats differ, src format is db3 while dest format is xml */
+			EXmlHash *xmlhash;
+			EDbHash *dbhash;
+			struct stat st;
+			
+			filename = g_build_filename (old_path, dent->d_name, NULL);
+			if (stat (filename, &st) == -1) {
+				g_free (filename);
+				continue;
+			}
+			
+			dbhash = e_dbhash_new (filename);
+			g_free (filename);
+			
+			filename = g_strdup_printf ("%s/%s.ics-%s", new_path, component, dent->d_name);
+			if (stat (filename, &st) != -1)
+				unlink (filename);
+			xmlhash = e_xmlhash_new (filename);
+			g_free (filename);
+			
+			e_dbhash_foreach_key (dbhash, migrate_pilot_db_key, xmlhash);
+			
+			e_dbhash_destroy (dbhash);
+			
+			e_xmlhash_write (xmlhash);
+			e_xmlhash_destroy (xmlhash);
+		}
+	}
+	
+	closedir (dir);
+}
+
 gboolean
-migrate_calendars (CalendarComponent *component, int major, int minor, int revision)
+migrate_calendars (CalendarComponent *component, int major, int minor, int revision, GError **err)
 {
 	ESourceGroup *on_this_computer = NULL, *on_the_web = NULL, *contacts = NULL;
 	ESource *personal_source = NULL;
-	gboolean retval = TRUE;
+	gboolean retval = FALSE;
 
 	/* we call this unconditionally now - create_groups either
 	   creates the groups/sources or it finds the necessary
@@ -572,8 +731,9 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 			xmlFreeDoc(config_doc);
 
 			if (res != 0) {
-				g_warning("Could not move config from bonobo-conf to gconf");
-				return FALSE;
+				/* FIXME: set proper domain/code */
+				g_set_error(err, 0, 0, _("Unable to migrate old settings from evolution/config.xmldb"));
+				goto fail;
 			}
 		}
 
@@ -599,8 +759,12 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 
 				source_name = get_source_name (on_this_computer, (char*)l->data);
 
-				if (!migrate_ical_folder (l->data, on_this_computer, source_name, E_CAL_SOURCE_TYPE_EVENT))
-					retval = FALSE;
+				if (!migrate_ical_folder (l->data, on_this_computer, source_name, E_CAL_SOURCE_TYPE_EVENT)) {
+					/* FIXME: domain/code */
+					g_set_error(err, 0, 0, _("Unable to migrate calendar `%s'"), source_name);
+					g_free(source_name);
+					goto fail;
+				}
 				
 				g_free (source_name);
 			}
@@ -626,18 +790,51 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 			
 			for (i = 0; keys[i]; i++) {
 				gconf_val = gconf_client_get (gconf, keys[i], NULL);
-				if (gconf_val->type != GCONF_VALUE_INT)
-					gconf_client_unset (gconf, keys[i], NULL);
-				gconf_value_free (gconf_val);
+				if (gconf_val) {
+					if (gconf_val->type != GCONF_VALUE_INT)
+						gconf_client_unset (gconf, keys[i], NULL);
+					gconf_value_free (gconf_val);
+				}
 			}
 			
 			g_object_unref (gconf);
 		}
+		
+		if (minor < 5 || (minor == 5 && revision <= 10)) {
+			char *old_path, *new_path;
+			
+			old_path = g_build_filename (g_get_home_dir (), "evolution", "local", "Calendar", NULL);
+			new_path = g_build_filename (calendar_component_peek_base_directory (component),
+						     "calendar", "local", "system", NULL);
+			migrate_pilot_data ("calendar", "calendar", old_path, new_path);
+			g_free (new_path);
+			g_free (old_path);
+		}
+
+		/* we only need to do this next step if people ran
+		   older versions of 1.5.  We need to clear out the
+		   absolute URI's that were assigned to ESources
+		   during one phase of development, as they take
+		   precedent over relative uris (but aren't updated
+		   when editing an ESource). */
+		if (minor == 5 && revision <= 11) {
+			GSList *g;
+			for (g = e_source_list_peek_groups (calendar_component_peek_source_list (component)); g; g = g->next) {
+				ESourceGroup *group = g->data;
+				GSList *s;
+
+				for (s = e_source_group_peek_sources (group); s; s = s->next) {
+					ESource *source = s->data;
+					e_source_set_absolute_uri (source, NULL);
+				}
+			}
+		}
 
 	}
 
 	e_source_list_sync (calendar_component_peek_source_list (component), NULL);
-
+	retval = TRUE;
+fail:
 	if (on_this_computer)
 		g_object_unref (on_this_computer);
 	if (on_the_web)
@@ -651,12 +848,12 @@ migrate_calendars (CalendarComponent *component, int major, int minor, int revis
 }
 
 gboolean
-migrate_tasks (TasksComponent *component, int major, int minor, int revision)
+migrate_tasks (TasksComponent *component, int major, int minor, int revision, GError **err)
 {
 	ESourceGroup *on_this_computer = NULL;
 	ESourceGroup *on_the_web = NULL;
 	ESource *personal_source = NULL;
-	gboolean retval = TRUE;
+	gboolean retval = FALSE;
 
 	/* we call this unconditionally now - create_groups either
 	   creates the groups/sources or it finds the necessary
@@ -687,21 +884,21 @@ migrate_tasks (TasksComponent *component, int major, int minor, int revision)
 			xmlFreeDoc(config_doc);
 
 			if (res != 0) {
-				g_warning("Could not move config from bonobo-conf to gconf");
-				return FALSE;
+				g_set_error(err, 0, 0, _("Unable to migrate old settings from evolution/config.xmldb"));
+				goto fail;
 			}
 		}
 
 		if (minor <= 4) {
 			GSList *migration_dirs, *l;
 			char *path, *local_task_folder;
-
+			
 			setup_progress_dialog (TRUE);
 			
 			path = g_build_filename (g_get_home_dir (), "evolution", "local", NULL);
 			migration_dirs = e_folder_map_local_folders (path, "tasks");
 			local_task_folder = g_build_filename (path, "Tasks", NULL);
-			g_free (path);			
+			g_free (path);
 
 			if (personal_source)
 				migrate_ical_folder_to_source (local_task_folder, personal_source, E_CAL_SOURCE_TYPE_TODO);
@@ -714,8 +911,12 @@ migrate_tasks (TasksComponent *component, int major, int minor, int revision)
 
 				source_name = get_source_name (on_this_computer, (char*)l->data);
 
-				if (!migrate_ical_folder (l->data, on_this_computer, source_name, E_CAL_SOURCE_TYPE_TODO))
-					retval = FALSE;
+				if (!migrate_ical_folder (l->data, on_this_computer, source_name, E_CAL_SOURCE_TYPE_TODO)) {
+					/* FIXME: domain/code */
+					g_set_error(err, 0, 0, _("Unable to migrate tasks `%s'"), source_name);
+					g_free(source_name);
+					goto fail;
+				}
 				
 				g_free (source_name);
 			}
@@ -723,11 +924,42 @@ migrate_tasks (TasksComponent *component, int major, int minor, int revision)
 			g_free (local_task_folder);
 
 			dialog_close ();
-		}		
+		}
+		
+		if (minor < 5 || (minor == 5 && revision <= 10)) {
+			char *old_path, *new_path;
+			
+			old_path = g_build_filename (g_get_home_dir (), "evolution", "local", "Tasks", NULL);
+			new_path = g_build_filename (tasks_component_peek_base_directory (component),
+						     "tasks", "local", "system", NULL);
+			migrate_pilot_data ("tasks", "todo", old_path, new_path);
+			g_free (new_path);
+			g_free (old_path);
+		}
+
+		/* we only need to do this next step if people ran
+		   older versions of 1.5.  We need to clear out the
+		   absolute URI's that were assigned to ESources
+		   during one phase of development, as they take
+		   precedent over relative uris (but aren't updated
+		   when editing an ESource). */
+		if (minor == 5 && revision <= 11) {
+			GSList *g;
+			for (g = e_source_list_peek_groups (tasks_component_peek_source_list (component)); g; g = g->next) {
+				ESourceGroup *group = g->data;
+				GSList *s;
+
+				for (s = e_source_group_peek_sources (group); s; s = s->next) {
+					ESource *source = s->data;
+					e_source_set_absolute_uri (source, NULL);
+				}
+			}
+		}
 	}
 
 	e_source_list_sync (tasks_component_peek_source_list (component), NULL);
-
+	retval = TRUE;
+fail:
 	if (on_this_computer)
 		g_object_unref (on_this_computer);
 	if (on_the_web)
