@@ -46,8 +46,6 @@
 #include "camel-stream.h"
 #include "camel-seekable-stream.h"
 
-#include "e-util/e-memory.h"
-
 #define r(x) 
 #define h(x) 
 #define c(x) 
@@ -59,6 +57,8 @@
 
 #define MEMPOOL
 
+#define STRUCT_ALIGN 4
+
 #ifdef PURIFY
 int inend_id = -1,
   inbuffer_id = -1;
@@ -71,6 +71,115 @@ int inend_id = -1,
 #define _header_scan_state _CamelMimeParserPrivate
 #define _PRIVATE(o) (((CamelMimeParser *)(o))->priv)
 
+#ifdef MEMPOOL
+typedef struct _MemPoolNode {
+	struct _MemPoolNode *next;
+
+	int free;
+	char data[1];
+} MemPoolNode;
+
+typedef struct _MemPoolThresholdNode {
+	struct _MemPoolThresholdNode *next;
+	char data[1];
+} MemPoolThresholdNode;
+
+typedef struct _MemPool {
+	int blocksize;
+	int threshold;
+	struct _MemPoolNode *blocks;
+	struct _MemPoolThresholdNode *threshold_blocks;
+} MemPool;
+
+MemPool *mempool_new(int blocksize, int threshold);
+void *mempool_alloc(MemPool *pool, int size);
+void mempool_flush(MemPool *pool, int freeall);
+void mempool_free(MemPool *pool);
+
+MemPool *mempool_new(int blocksize, int threshold)
+{
+	MemPool *pool;
+
+	pool = g_malloc(sizeof(*pool));
+	if (threshold >= blocksize)
+		threshold = blocksize * 2 / 3;
+	pool->blocksize = blocksize;
+	pool->threshold = threshold;
+	pool->blocks = NULL;
+	pool->threshold_blocks = NULL;
+	return pool;
+}
+
+void *mempool_alloc(MemPool *pool, int size)
+{
+	size = (size + STRUCT_ALIGN) & (~(STRUCT_ALIGN-1));
+	if (size>=pool->threshold) {
+		MemPoolThresholdNode *n;
+
+		n = g_malloc(sizeof(*n) - sizeof(char) + size);
+		n->next = pool->threshold_blocks;
+		pool->threshold_blocks = n;
+		return &n->data[0];
+	} else {
+		MemPoolNode *n;
+
+		n = pool->blocks;
+		while (n) {
+			if (n->free >= size) {
+				n->free -= size;
+				return &n->data[n->free];
+			}
+			n = n->next;
+		}
+
+		n = g_malloc(sizeof(*n) - sizeof(char) + pool->blocksize);
+		n->next = pool->blocks;
+		pool->blocks = n;
+		n->free = pool->blocksize - size;
+		return &n->data[n->free];
+	}
+}
+
+void mempool_flush(MemPool *pool, int freeall)
+{
+	MemPoolThresholdNode *tn, *tw;
+	MemPoolNode *pw, *pn;
+
+	tw = pool->threshold_blocks;
+	while (tw) {
+		tn = tw->next;
+		g_free(tw);
+		tw = tn;
+	}
+	pool->threshold_blocks = NULL;
+
+	if (freeall) {
+		pw = pool->blocks;
+		while (pw) {
+			pn = pw->next;
+			g_free(pw);
+			pw = pn;
+		}
+		pool->blocks = NULL;
+	} else {
+		pw = pool->blocks;
+		while (pw) {
+			pw->free = pool->blocksize;
+			pw = pw->next;
+		}
+	}
+}
+
+void mempool_free(MemPool *pool)
+{
+	if (pool) {
+		mempool_flush(pool, 1);
+		g_free(pool);
+	}
+}
+
+#endif
+
 struct _header_scan_state {
 
     /* global state */
@@ -104,7 +213,6 @@ struct _header_scan_state {
 	unsigned int eof:1;		/* reached eof? */
 
 	off_t start_of_from;	/* where from started */
-	off_t start_of_boundary; /* where the last boundary started */
 	off_t start_of_headers;	/* where headers started from the last scan */
 
 	off_t header_start;	/* start of last header, or -1 */
@@ -124,7 +232,7 @@ struct _header_scan_stack {
 	enum _camel_mime_parser_state savestate; /* state at invocation of this part */
 
 #ifdef MEMPOOL
-	EMemPool *pool;		/* memory pool to keep track of headers/etc at this level */
+	MemPool *pool;		/* memory pool to keep track of headers/etc at this level */
 #endif
 	struct _camel_header_raw *headers;	/* headers for this part */
 
@@ -161,8 +269,6 @@ static int folder_scan_skip_line(struct _header_scan_state *s, GByteArray *save)
 static off_t folder_seek(struct _header_scan_state *s, off_t offset, int whence);
 static off_t folder_tell(struct _header_scan_state *s);
 static int folder_read(struct _header_scan_state *s);
-static void folder_push_part(struct _header_scan_state *s, struct _header_scan_stack *h);
-
 #ifdef MEMPOOL
 static void header_append_mempool(struct _header_scan_state *s, struct _header_scan_stack *h, char *header, int offset);
 #endif
@@ -749,24 +855,6 @@ camel_mime_parser_tell_start_from (CamelMimeParser *parser)
 	return s->start_of_from;
 }
 
-/**
- * camel_mime_parser_tell_start_boundary:
- * @parser: MIME parser object
- * 
- * When parsing a multipart, this returns the start of the last
- * boundary.
- * 
- * Return value: The start of the boundary, or -1 if there
- * was no boundary encountered yet.
- **/
-off_t
-camel_mime_parser_tell_start_boundary(CamelMimeParser *parser)
-{
-	struct _header_scan_state *s = _PRIVATE (parser);
-
-	return s->start_of_boundary;
-}
-
 /**
  * camel_mime_parser_seek:
  * @parser: MIME parser object
@@ -808,30 +896,6 @@ camel_mime_parser_state (CamelMimeParser *parser)
 	return s->state;
 }
 
-/**
- * camel_mime_parser_push_state:
- * @mp: MIME parser object
- * @newstate: New state
- * @boundary: Boundary marker for state.
- * 
- * Pre-load a new parser state.  Used to post-parse multipart content
- * without headers.
- **/
-void
-camel_mime_parser_push_state(CamelMimeParser *mp, enum _camel_mime_parser_state newstate, const char *boundary)
-{
-	struct _header_scan_stack *h;
-	struct _header_scan_state *s = _PRIVATE(mp);
-
-	h = g_malloc0(sizeof(*h));
-	h->boundarylen = strlen(boundary)+2;
-	h->boundarylenfinal = h->boundarylen+2;
-	h->boundary = g_malloc(h->boundarylen+3);
-	sprintf(h->boundary, "--%s--", boundary);
-	folder_push_part(s, h);
-	s->state = newstate;
-}
-
 /**
  * camel_mime_parser_stream:
  * @parser: MIME parser object
@@ -1003,7 +1067,7 @@ folder_pull_part(struct _header_scan_state *s)
 		s->parts = h->parent;
 		g_free(h->boundary);
 #ifdef MEMPOOL
-		e_mempool_destroy(h->pool);
+		mempool_free(h->pool);
 #else
 		camel_header_raw_clear(&h->headers);
 #endif
@@ -1111,18 +1175,18 @@ header_append_mempool(struct _header_scan_state *s, struct _header_scan_stack *h
 	content = strchr(header, ':');
 	if (content) {
 		register int len;
-		n = e_mempool_alloc(h->pool, sizeof(*n));
+		n = mempool_alloc(h->pool, sizeof(*n));
 		n->next = NULL;
 		
 		len = content-header;
-		n->name = e_mempool_alloc(h->pool, len+1);
+		n->name = mempool_alloc(h->pool, len+1);
 		memcpy(n->name, header, len);
 		n->name[len] = 0;
 		
 		content++;
 		
 		len = s->outptr - content;
-		n->value = e_mempool_alloc(h->pool, len+1);
+		n->value = mempool_alloc(h->pool, len+1);
 		memcpy(n->value, content, len);
 		n->value[len] = 0;
 		
@@ -1182,7 +1246,7 @@ folder_scan_header(struct _header_scan_state *s, int *lastone)
 
 	h = g_malloc0(sizeof(*h));
 #ifdef MEMPOOL
-	h->pool = e_mempool_new(8192, 4096, E_MEMPOOL_ALIGN_STRUCT);
+	h->pool = mempool_new(8192, 4096);
 #endif
 
 	if (s->parts)
@@ -1454,7 +1518,6 @@ folder_scan_init(void)
 
 	s->start_of_from = -1;
 	s->start_of_headers = -1;
-	s->start_of_boundary = -1;
 
 	s->midline = FALSE;
 	s->scan_from = FALSE;
@@ -1526,7 +1589,8 @@ folder_scan_step(struct _header_scan_state *s, char **databuffer, size_t *datale
 	struct _header_scan_stack *h, *hb;
 	const char *content;
 	const char *bound;
-	int type, state, seenlast;
+	int type;
+	int state;
 	CamelContentType *ct = NULL;
 	struct _header_scan_filter *f;
 	size_t presize;
@@ -1611,7 +1675,7 @@ tail_recurse:
 		type = CAMEL_MIME_PARSER_STATE_HEADER;
 		if ( (content = camel_header_raw_find(&h->headers, "Content-Type", NULL))
 		     && (ct = camel_content_type_decode(content))) {
-			if (!g_ascii_strcasecmp(ct->type, "multipart")) {
+			if (!strcasecmp(ct->type, "multipart")) {
 				if (!camel_content_type_is(ct, "multipart", "signed")
 				    && (bound = camel_content_type_param(ct, "boundary"))) {
 					d(printf("multipart, boundary = %s\n", bound));
@@ -1630,7 +1694,7 @@ tail_recurse:
 			} else if (!strcasecmp(ct->type, "message")) {
 				if (!strcasecmp(ct->subtype, "rfc822")
 				    || !strcasecmp(ct->subtype, "news")
-				    /*|| !g_ascii_strcasecmp(ct->subtype, "partial")*/) {
+				    /*|| !strcasecmp(ct->subtype, "partial")*/) {
 					type = CAMEL_MIME_PARSER_STATE_MESSAGE;
 				}
 			}
@@ -1696,14 +1760,6 @@ tail_recurse:
 		
 	case CAMEL_MIME_PARSER_STATE_MULTIPART:
 		h = s->parts;
-		/* This mess looks for the next boundary on this
-		level.  Once it finds the last one, it keeps going,
-		looking for post-multipart content ('postface').
-		Because messages might have duplicate boundaries for
-		different parts, it makes sure it stops if its already
-		found an end boundary for this part.  It handles
-		truncated and missing boundaries appropriately too. */
-		seenlast = FALSE;
 		do {
 			do {
 				hb = folder_scan_content(s, &state, databuffer, datalength);
@@ -1724,17 +1780,15 @@ tail_recurse:
 				}
 			} while (hb==h && *datalength>0);
 			h->prestage++;
-			if (*datalength==0 && hb==h && !seenlast) {
-				d(printf("got boundary: %s last=%d\n", hb->boundary, state));
-				s->start_of_boundary = folder_tell(s);
+			if (*datalength==0 && hb==h) {
+				d(printf("got boundary: %s\n", hb->boundary));
 				folder_scan_skip_line(s, NULL);
 				if (!state) {
 					s->state = CAMEL_MIME_PARSER_STATE_FROM;
 					folder_scan_step(s, databuffer, datalength);
 					s->parts->savestate = CAMEL_MIME_PARSER_STATE_MULTIPART; /* set return state for the new head part */
 					return;
-				} else
-					seenlast = TRUE;
+				}
 			} else {
 				break;
 			}
@@ -1852,7 +1906,7 @@ int main(int argc, char **argv)
 			case CAMEL_MIME_PARSER_STATE_HEADER:
 				if (s->parts->content_type
 				    && (charset = camel_content_type_param(s->parts->content_type, "charset"))) {
-					if (g_ascii_strcasecmp(charset, "us-ascii")) {
+					if (strcasecmp(charset, "us-ascii")) {
 #if 0
 						folder_push_filter_charset(s, "UTF-8", charset);
 #endif
