@@ -28,6 +28,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <semaphore.h>
+#include <errno.h>
 
 #include "camel-object.h"
 #include "camel-file-utils.h"
@@ -42,8 +43,10 @@
 /* I just mashed the keyboard for these... */
 #define CAMEL_OBJECT_MAGIC           	 0x77A344ED
 #define CAMEL_OBJECT_CLASS_MAGIC     	 0xEE26A997
+#define CAMEL_INTERFACE_MAGIC     	 0xBCE137A7
 #define CAMEL_OBJECT_FINALISED_MAGIC       0x84AC365F
 #define CAMEL_OBJECT_CLASS_FINALISED_MAGIC 0x7621ABCD
+#define CAMEL_INTERFACE_FINALISED_MAGIC    0x7CB2FE71
 
 /* ** Quickie type system ************************************************* */
 
@@ -112,6 +115,9 @@ static CamelHookPair *co_metadata_pair(CamelObject *obj, int create);
 static const char *meta_name = "object:meta";
 #define CAMEL_OBJECT_STATE_FILE_MAGIC "CLMD"
 
+/* interface stuff */
+static const char *interface_name = "object:interface";
+
 /* ********************************************************************** */
 
 static CamelHookList *camel_object_get_hooks(CamelObject *o);
@@ -134,7 +140,9 @@ static EMutex *type_lock;
 static GHashTable *type_table;
 static EMemChunk *type_chunks;
 
+/* fundamental types are accessed via global */
 CamelType camel_object_type;
+CamelType camel_interface_type;
 
 #define P_LOCK(l) (pthread_mutex_lock(&l))
 #define P_UNLOCK(l) (pthread_mutex_unlock(&l))
@@ -143,7 +151,6 @@ CamelType camel_object_type;
 #define CLASS_LOCK(k) (g_mutex_lock((((CamelObjectClass *)k)->lock)))
 #define CLASS_UNLOCK(k) (g_mutex_unlock((((CamelObjectClass *)k)->lock)))
 
-
 static struct _CamelHookPair *
 pair_alloc(void)
 {
@@ -210,9 +217,11 @@ camel_type_init(void)
 
 /* ************************************************************************ */
 
+/* CamelObject base methods */
+
 /* Should this return the object to the caller? */
 static void
-cobject_init (CamelObject *o, CamelObjectClass *klass)
+cobject_init(CamelObject *o, CamelObjectClass *klass)
 {
 	o->klass = klass;
 	o->magic = CAMEL_OBJECT_MAGIC;
@@ -434,13 +443,15 @@ cobject_state_read(CamelObject *obj, FILE *fp)
 		CamelArgV *argv;
 
 		if (camel_file_util_decode_uint32(fp, &count) == -1
-			|| count == 0) {
+			|| count == 0 || count > 1024) {
 			/* maybe it was just version 0 afterall */
 			return 0;
 		}
-
+		
 		/* we batch up the properties and set them in one go */
-		argv = g_malloc(sizeof(*argv) + (count - CAMEL_ARGV_MAX) * sizeof(argv->argv[0]));
+		if (!(argv = g_try_malloc (sizeof (*argv) + (count - CAMEL_ARGV_MAX) * sizeof (argv->argv[0]))))
+			return -1;
+		
 		argv->argc = 0;
 		for (i=0;i<count;i++) {
 			if (camel_file_util_decode_uint32(fp, &argv->argv[argv->argc].tag) == -1)
@@ -505,8 +516,8 @@ cobject_state_write(CamelObject *obj, FILE *fp)
 
 	scan = meta;
 	while (scan) {
-		if (camel_file_util_encode_string(fp, meta->name) == -1
-		    || camel_file_util_encode_string(fp, meta->value) == -1)
+		if (camel_file_util_encode_string(fp, scan->name) == -1
+		    || camel_file_util_encode_string(fp, scan->value) == -1)
 			goto abort;
 		scan = scan->next;
 	}
@@ -607,8 +618,61 @@ cobject_class_finalise(CamelObjectClass * klass)
 	g_free(klass);
 }
 
+
+/* CamelInterface base methods */
+
+static void
+cinterface_init(CamelObject *o, CamelObjectClass *klass)
+{
+	g_error("Cannot instantiate interfaces, trying to instantiate '%s'", klass->name);
+	abort();
+}
+
+static int
+cinterface_getv(CamelObject *o, CamelException *ex, CamelArgGetV *args)
+{
+	return 0;
+}
+
+static int
+cinterface_setv(CamelObject *o, CamelException *ex, CamelArgV *args)
+{
+	return 0;
+}
+
+static void
+cinterface_free(CamelObject *o, guint32 tag, void *value)
+{
+	/* NOOP */
+}
+
+static void
+cinterface_class_init(CamelObjectClass *klass)
+{
+	klass->magic = CAMEL_INTERFACE_MAGIC;
+
+	/* just setup dummy callbacks, properties could be part of the interface but we support none */
+	klass->getv = cinterface_getv;
+	klass->setv = cinterface_setv;
+	klass->free = cinterface_free;
+
+	/* TODO: ok, these are cruft hanging around an interface, but it saves having to define two different class bases */
+	klass->meta_get = NULL;
+	klass->meta_set = NULL;
+	klass->state_read = NULL;
+	klass->state_write = NULL;
+}
+
+static void
+cinterface_class_finalise(CamelObjectClass * klass)
+{
+	klass->magic = CAMEL_INTERFACE_FINALISED_MAGIC;
+	g_free(klass);
+}
+
+/* this function must be called for any other in the object system */
 CamelType
-camel_object_get_type (void)
+camel_object_get_type(void)
 {
 	if (camel_object_type == CAMEL_INVALID_TYPE) {
 		camel_type_init();
@@ -617,6 +681,12 @@ camel_object_get_type (void)
 							sizeof(CamelObject), sizeof(CamelObjectClass),
 							cobject_class_init, cobject_class_finalise,
 							cobject_init, cobject_finalise);
+
+		camel_interface_type = camel_type_register(NULL, "CamelInterface",
+							   0, sizeof(CamelInterface),
+							   cinterface_class_init, cinterface_class_finalise,
+							   cinterface_init, NULL);
+							   
 	}
 
 	return camel_object_type;
@@ -632,24 +702,19 @@ camel_type_class_init(CamelObjectClass *klass, CamelObjectClass *type)
 		type->klass_init(klass);
 }
 
-CamelType
-camel_type_register (CamelType parent, const char * name,
-		     /*unsigned int ver, unsigned int rev,*/
-		     size_t object_size, size_t klass_size,
-		     CamelObjectClassInitFunc class_init,
-		     CamelObjectClassFinalizeFunc class_finalise,
-		     CamelObjectInitFunc object_init,
-		     CamelObjectFinalizeFunc object_finalise)
+static CamelType
+co_type_register(CamelType parent, const char * name,
+		 /*unsigned int ver, unsigned int rev,*/
+		 size_t object_size, size_t klass_size,
+		 CamelObjectClassInitFunc class_init,
+		 CamelObjectClassFinalizeFunc class_finalise,
+		 CamelObjectInitFunc object_init,
+		 CamelObjectFinalizeFunc object_finalise)
 {
 	CamelObjectClass *klass;
 	/*int offset;
 	  size_t size;*/
 
-	if (parent != NULL && parent->magic != CAMEL_OBJECT_CLASS_MAGIC) {
-		g_warning("camel_type_register: invalid junk parent class for '%s'", name);
-		return NULL;
-	}
-
 	E_LOCK(type_lock);
 
 	/* Have to check creation, it might've happened in another thread before we got here */
@@ -721,6 +786,37 @@ camel_type_register (CamelType parent, const char * name,
 	return klass;
 }
 
+CamelType
+camel_type_register(CamelType parent, const char * name,
+		    /*unsigned int ver, unsigned int rev,*/
+		    size_t object_size, size_t klass_size,
+		    CamelObjectClassInitFunc class_init,
+		    CamelObjectClassFinalizeFunc class_finalise,
+		    CamelObjectInitFunc object_init,
+		    CamelObjectFinalizeFunc object_finalise)
+{
+	if (parent != NULL && parent->magic != CAMEL_OBJECT_CLASS_MAGIC) {
+		g_warning("camel_type_register: invalid junk parent class for '%s'", name);
+		return NULL;
+	}
+
+	return co_type_register(parent, name, object_size, klass_size, class_init, class_finalise, object_init, object_finalise);
+}
+
+CamelType
+camel_interface_register(CamelType parent, const char *name,
+			 size_t class_size,
+			 CamelObjectClassInitFunc class_init,
+			 CamelObjectClassFinalizeFunc class_finalise)
+{
+	if (parent != NULL && parent->magic != CAMEL_INTERFACE_MAGIC) {
+		g_warning("camel_interface_register: invalid junk parent class for '%s'", name);
+		return NULL;
+	}
+
+	return camel_type_register(parent, name, 0, class_size, class_init, class_finalise, NULL, NULL);
+}
+
 static void
 camel_object_init(CamelObject *o, CamelObjectClass *klass, CamelType type)
 {
@@ -841,7 +937,7 @@ camel_object_unref(void *vo)
 }
 
 const char *
-camel_type_to_name (CamelType type)
+camel_type_to_name(CamelType type)
 {
 	if (type == NULL)
 		return "(NULL class)";
@@ -849,6 +945,9 @@ camel_type_to_name (CamelType type)
 	if (type->magic == CAMEL_OBJECT_CLASS_MAGIC)
 		return type->name;
 
+	if (type->magic == CAMEL_INTERFACE_MAGIC)
+		return type->name;
+
 	return "(Junk class)";
 }
 
@@ -860,7 +959,7 @@ CamelType camel_name_to_type(const char *name)
 }
 
 static char *
-desc_data(CamelObject *o, int ok)
+desc_data(CamelObject *o, guint32 ok)
 {
 	char *what;
 
@@ -872,27 +971,36 @@ desc_data(CamelObject *o, int ok)
 		what = g_strdup_printf("CLASS '%s'", ((CamelObjectClass *)o)->name);
 	else if (o->magic == CAMEL_OBJECT_CLASS_MAGIC)
 		what = g_strdup_printf("CLASS '%s'", ((CamelObjectClass *)o)->name);
+	else if (o->magic == CAMEL_INTERFACE_MAGIC)
+		what = g_strdup_printf("INTERFACE '%s'", ((CamelObjectClass *)o)->name);
 	else if (o->magic == CAMEL_OBJECT_FINALISED_MAGIC)
 		what = g_strdup_printf("finalised OBJECT");
 	else if (o->magic == CAMEL_OBJECT_CLASS_FINALISED_MAGIC)
 		what = g_strdup_printf("finalised CLASS");
+	else if (o->magic == CAMEL_INTERFACE_FINALISED_MAGIC)
+		what = g_strdup_printf("finalised INTERFACE");
 	else 
 		what = g_strdup_printf("junk data");
 
 	return what;
 }
 
+#define check_magic(o, ctype, omagic) \
+	( ((CamelObject *)(o))->magic == (omagic) \
+	&& (ctype)->magic == CAMEL_OBJECT_CLASS_MAGIC) \
+	? 1 : check_magic_fail(o, ctype, omagic)
+
 static gboolean
-check_magic(void *o, CamelType ctype, int isob)
+check_magic_fail(void *o, CamelType ctype, guint32 omagic)
 {
 	char *what, *to;
 
-	what = desc_data(o, isob?CAMEL_OBJECT_MAGIC:CAMEL_OBJECT_CLASS_MAGIC);
+	what = desc_data(o, omagic);
 	to = desc_data((CamelObject *)ctype, CAMEL_OBJECT_CLASS_MAGIC);
 
 	if (what || to) {
 		if (what == NULL) {
-			if (isob)
+			if (omagic == CAMEL_OBJECT_MAGIC)
 				what = g_strdup_printf("OBJECT '%s'", ((CamelObject *)o)->klass->name);
 			else
 				what = g_strdup_printf("OBJECT '%s'", ((CamelObjectClass *)o)->name);
@@ -910,11 +1018,11 @@ check_magic(void *o, CamelType ctype, int isob)
 }
 
 gboolean
-camel_object_is (CamelObject *o, CamelType ctype)
+camel_object_is(CamelObject *o, CamelType ctype)
 {
 	CamelObjectClass *k;
 
-	g_return_val_if_fail(check_magic(o, ctype, TRUE), FALSE);
+	g_return_val_if_fail(check_magic(o, ctype, CAMEL_OBJECT_MAGIC), FALSE);
 
 	k = o->klass;
 	while (k) {
@@ -927,9 +1035,23 @@ camel_object_is (CamelObject *o, CamelType ctype)
 }
 
 gboolean
-camel_object_class_is (CamelObjectClass *k, CamelType ctype)
+camel_object_class_is(CamelObjectClass *k, CamelType ctype)
 {
-	g_return_val_if_fail(check_magic(k, ctype, FALSE), FALSE);
+	g_return_val_if_fail(check_magic(k, ctype, CAMEL_OBJECT_CLASS_MAGIC), FALSE);
+
+	while (k) {
+		if (k == ctype)
+			return TRUE;
+		k = k->parent;
+	}
+
+	return FALSE;
+}
+
+gboolean
+camel_interface_is(CamelObjectClass *k, CamelType ctype)
+{
+	g_return_val_if_fail(check_magic(k, ctype, CAMEL_INTERFACE_MAGIC), FALSE);
 
 	while (k) {
 		if (k == ctype)
@@ -945,7 +1067,7 @@ camel_object_cast(CamelObject *o, CamelType ctype)
 {
 	CamelObjectClass *k;
 
-	g_return_val_if_fail(check_magic(o, ctype, TRUE), NULL);
+	g_return_val_if_fail(check_magic(o, ctype, CAMEL_OBJECT_MAGIC), NULL);
 
 	k = o->klass;
 	while (k) {
@@ -964,7 +1086,7 @@ camel_object_class_cast(CamelObjectClass *k, CamelType ctype)
 {
 	CamelObjectClass *r = k;
 
-	g_return_val_if_fail(check_magic(k, ctype, FALSE), NULL);
+	g_return_val_if_fail(check_magic(k, ctype, CAMEL_OBJECT_CLASS_MAGIC), NULL);
 
 	while (k) {
 		if (k == ctype)
@@ -977,6 +1099,55 @@ camel_object_class_cast(CamelObjectClass *k, CamelType ctype)
 	return NULL;
 }
 
+CamelObjectClass *
+camel_interface_cast(CamelObjectClass *k, CamelType ctype)
+{
+	CamelObjectClass *r = k;
+
+	g_return_val_if_fail(check_magic(k, ctype, CAMEL_INTERFACE_MAGIC), NULL);
+
+	while (k) {
+		if (k == ctype)
+			return r;
+		k = k->parent;
+	}
+
+	g_warning("Interface '%s' doesn't have '%s' in its hierarchy", r->name, ctype->name);
+
+	return NULL;
+}
+
+static CamelHookPair *
+co_find_pair(CamelObjectClass *klass, const char *name)
+{
+	CamelHookPair *hook;
+
+	hook = klass->hooks;
+	while (hook) {
+		if (strcmp(hook->name, name) == 0)
+			return hook;
+		hook = hook->next;
+	}
+
+	return NULL;
+}
+
+static CamelHookPair *
+co_find_pair_ptr(CamelObjectClass *klass, const char *name)
+{
+	CamelHookPair *hook;
+
+	hook = klass->hooks;
+	while (hook) {
+		if (hook->name == name)
+			return hook;
+		hook = hook->next;
+	}
+
+	return NULL;
+}
+
+/* class functions */
 void
 camel_object_class_add_event(CamelObjectClass *klass, const char *name, CamelObjectEventPrepFunc prep)
 {
@@ -984,14 +1155,17 @@ camel_object_class_add_event(CamelObjectClass *klass, const char *name, CamelObj
 
 	g_return_if_fail (name);
 
-	pair = klass->hooks;
-	while (pair) {
-		if (strcmp(pair->name, name) == 0) {
-			g_warning("camel_object_class_add_event: `%s' is already declared for '%s'\n",
-				  name, klass->name);
-			return;
-		}
-		pair = pair->next;
+	pair = co_find_pair(klass, name);
+	if (pair) {
+		g_warning("camel_object_class_add_event: `%s' is already declared for '%s'",
+			  name, klass->name);
+		return;
+	}
+
+	if (klass->magic == CAMEL_INTERFACE_MAGIC && prep != NULL) {
+		g_warning("camel_object_class_add_event: `%s', CamelInterface '%s' may not have an event prep function - ignored",
+			  name, klass->name);
+		prep = NULL;
 	}
 
 	pair = pair_alloc();
@@ -1003,9 +1177,53 @@ camel_object_class_add_event(CamelObjectClass *klass, const char *name, CamelObj
 	klass->hooks = pair;
 }
 
+void
+camel_object_class_add_interface(CamelObjectClass *klass, CamelType itype)
+{
+	CamelHookPair *pair;
+	CamelType iscan;
+	GPtrArray *interfaces;
+	int i;
+
+	if (!camel_interface_is(itype, camel_interface_type)) {
+		g_warning("Cannot add an interface not derived from CamelInterface on class '%s'", klass->name);
+		return;
+	}
+
+	if (camel_object_class_is(klass, camel_interface_type)) {
+		g_warning("Cannot add an interface onto a class derived from CamelInterface");
+		return;
+	}
+
+	/* we store it on the class hooks so we don't have to add any extra space to the class */
+	pair = co_find_pair_ptr(klass, interface_name);
+	if (pair == NULL) {
+		pair = pair_alloc();
+		pair->data = g_ptr_array_new();
+		pair->next = klass->hooks;
+		klass->hooks = pair;
+	}
+
+	/* We just check that this type isn't added/derived anywhere else */
+	interfaces = pair->data;
+	iscan = itype;
+	while (iscan && iscan != camel_interface_type) {
+		for (i=0;i<interfaces->len;i++) {
+			if (camel_interface_is((CamelType)interfaces->pdata[i], iscan)) {
+				g_warning("Cannot add an interface twice '%s' on class '%s'\n", itype->name, klass->name);
+				return;
+			}
+		}
+		iscan = iscan->parent;
+	}
+
+	if (iscan == camel_interface_type)
+		g_ptr_array_add(interfaces, itype);
+}
+
 /* free hook data */
 static void
-camel_object_free_hooks (CamelObject *o)
+camel_object_free_hooks(CamelObject *o)
 {
 	CamelHookPair *pair, *next;
 
@@ -1033,7 +1251,7 @@ camel_object_free_hooks (CamelObject *o)
 
 /* return (allocate if required) the object's hook list, locking at the same time */
 static CamelHookList *
-camel_object_get_hooks (CamelObject *o)
+camel_object_get_hooks(CamelObject *o)
 {
 	static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 	CamelHookList *hooks;
@@ -1071,18 +1289,27 @@ camel_object_hook_event(void *vo, const char * name, CamelObjectEventHookFunc fu
 	g_return_val_if_fail(name != NULL, 0);
 	g_return_val_if_fail(func != NULL, 0);
 
-	hook = obj->klass->hooks;
-	while (hook) {
-		if (strcmp(hook->name, name) == 0)
-			goto setup;
-		hook = hook->next;
+	hook = co_find_pair(obj->klass, name);
+
+	/* Check all interfaces on this object for events defined on them */
+	if (hook == NULL) {
+		pair = co_find_pair_ptr(obj->klass, interface_name);
+		if (pair) {
+			GPtrArray *interfaces = pair->data;
+			int i;
+
+			for (i=0;i<interfaces->len;i++) {
+				hook = co_find_pair(interfaces->pdata[i], name);
+				if (hook)
+					goto setup;
+			}
+		}
+
+		g_warning("camel_object_hook_event: trying to hook event `%s' in class `%s' with no defined events.",
+			  name, obj->klass->name);
+
+		return 0;
 	}
-
-	g_warning("camel_object_hook_event: trying to hook event `%s' in class `%s' with no defined events.",
-		  name, obj->klass->name);
-
-	return 0;
-
 setup:
 	/* setup hook pair */
 	pair = pair_alloc();
@@ -1209,11 +1436,24 @@ camel_object_trigger_event(void *vo, const char * name, void *event_data)
 	g_return_if_fail (CAMEL_IS_OBJECT (obj));
 	g_return_if_fail (name);
 
-	hook = obj->klass->hooks;
-	while (hook) {
-		if (strcmp(hook->name, name) == 0)
-			goto trigger;
-		hook = hook->next;
+	hook = co_find_pair(obj->klass, name);
+	if (hook)
+		goto trigger;
+
+	if (obj->hooks == NULL)
+		return;
+
+	/* interface events can't have prep functions */
+	pair = co_find_pair_ptr(obj->klass, interface_name);
+	if (pair) {
+		GPtrArray *interfaces = pair->data;
+		int i;
+
+		for (i=0;i<interfaces->len;i++) {
+			hook = co_find_pair(interfaces->pdata[i], name);
+			if (hook)
+				goto trigger_interface;
+		}
 	}
 
 	g_warning("camel_object_trigger_event: trying to trigger unknown event `%s' in class `%s'",
@@ -1229,7 +1469,7 @@ trigger:
 	/* also, no hooks, dont bother going further */
 	if (obj->hooks == NULL)
 		return;
-
+trigger_interface:
 	/* lock the object for hook emission */
 	camel_object_ref(obj);
 	hooks = camel_object_get_hooks(obj);
@@ -1277,6 +1517,31 @@ trigger:
 	camel_object_unref(obj);
 }
 
+void *
+camel_object_get_interface(void *vo, CamelType itype)
+{
+	CamelObject *obj = vo;
+	CamelHookPair *pair;
+
+	g_return_val_if_fail(CAMEL_IS_OBJECT (obj), NULL);
+	g_return_val_if_fail(camel_interface_is(itype, camel_interface_type), NULL);
+
+	pair = co_find_pair_ptr(obj->klass, interface_name);
+	if (pair) {
+		GPtrArray *interfaces = pair->data;
+		int i;
+
+		for (i=0;i<interfaces->len;i++) {
+			if (camel_interface_is((CamelType)interfaces->pdata[i], itype))
+				return (CamelType)interfaces->pdata[i];
+		}
+	}
+
+	g_warning("Object %p class %s doesn't contain interface %s\n", vo, obj->klass->name, itype->name);
+
+	return NULL;
+}
+
 /* get/set arg methods */
 int camel_object_set(void *vo, CamelException *ex, ...)
 {
@@ -1327,6 +1592,55 @@ int camel_object_get(void *vo, CamelException *ex, ...)
 	return ret;
 }
 
+void *camel_object_get_ptr(void *vo, CamelException *ex, int tag)
+{
+	CamelObject *o = vo;
+	CamelArgGetV args;
+	CamelObjectClass *klass = o->klass;
+	int ret = 0;
+	void *val = NULL;
+
+	g_return_val_if_fail(CAMEL_IS_OBJECT(o), NULL);
+	g_return_val_if_fail((tag & CAMEL_ARG_TYPE) == CAMEL_ARG_OBJ
+			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_STR
+			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_PTR, 0);
+
+	/* woefully inefficient, *shrug */
+	args.argc = 1;
+	args.argv[0].tag = tag;
+	args.argv[0].ca_ptr = &val;
+
+	ret = klass->getv(o, ex, &args);
+	if (ret != 0)
+		return NULL;
+	else
+		return val;
+}
+
+int camel_object_get_int(void *vo, CamelException *ex, int tag)
+{
+	CamelObject *o = vo;
+	CamelArgGetV args;
+	CamelObjectClass *klass = o->klass;
+	int ret = 0;
+	int val = 0;
+
+	g_return_val_if_fail(CAMEL_IS_OBJECT(o), 0);
+	g_return_val_if_fail((tag & CAMEL_ARG_TYPE) == CAMEL_ARG_INT
+			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_BOO, 0);
+
+	/* woefully inefficient, *shrug */
+	args.argc = 1;
+	args.argv[0].tag = tag;
+	args.argv[0].ca_int = &val;
+
+	ret = klass->getv(o, ex, &args);
+	if (ret != 0)
+		return 0;
+	else
+		return val;
+}
+
 int camel_object_getv(void *vo, CamelException *ex, CamelArgGetV *args)
 {
 	g_return_val_if_fail(CAMEL_IS_OBJECT(vo), -1);
@@ -1512,7 +1826,7 @@ int camel_object_state_write(void *vo)
 {
 	CamelObject *obj = vo;
 	int res = -1;
-	char *file, *savename;
+	char *file, *savename, *tmp;
 	FILE *fp;
 
 	camel_object_get(vo, NULL, CAMEL_OBJECT_STATE_FILE, &file, NULL);
@@ -1520,6 +1834,12 @@ int camel_object_state_write(void *vo)
 		return 0;
 
 	savename = camel_file_util_savename(file);
+	tmp = strrchr(savename, '/');
+	if (tmp) {
+		*tmp = 0;
+		camel_mkdir(savename, 0777);
+		*tmp = '/';
+	}
 	fp = fopen(savename, "w");
 	if (fp != NULL) {
 		if (fwrite(CAMEL_OBJECT_STATE_FILE_MAGIC, 4, 1, fp) == 1
@@ -1531,6 +1851,8 @@ int camel_object_state_write(void *vo)
 		} else {
 			fclose(fp);
 		}
+	} else {
+		g_warning("Could not save object state file to '%s': %s", savename, g_strerror(errno));
 	}
 
 	g_free(savename);
@@ -1632,7 +1954,7 @@ camel_object_class_dump_tree(CamelType root)
  * Return value: 
  **/
 CamelObjectBag *
-camel_object_bag_new (GHashFunc hash, GEqualFunc equal, CamelCopyFunc keycopy, GFreeFunc keyfree)
+camel_object_bag_new(GHashFunc hash, GEqualFunc equal, CamelCopyFunc keycopy, GFreeFunc keyfree)
 {
 	CamelObjectBag *bag;
 
@@ -1654,7 +1976,7 @@ save_object(void *key, CamelObject *o, GPtrArray *objects)
 }
 
 void
-camel_object_bag_destroy (CamelObjectBag *bag)
+camel_object_bag_destroy(CamelObjectBag *bag)
 {
 	GPtrArray *objects = g_ptr_array_new();
 	int i;
@@ -1712,7 +2034,7 @@ co_bag_unreserve(CamelObjectBag *bag, const void *key)
  * previously been reserved using camel_object_bag_reserve().
  **/
 void
-camel_object_bag_add (CamelObjectBag *bag, const void *key, void *vo)
+camel_object_bag_add(CamelObjectBag *bag, const void *key, void *vo)
 {
 	CamelObject *o = vo;
 	CamelHookList *hooks;
@@ -1736,6 +2058,7 @@ camel_object_bag_add (CamelObjectBag *bag, const void *key, void *vo)
 	pair->name = bag_name;
 	pair->data = bag;
 	pair->flags = 0;
+	pair->func.event = NULL;
 
 	pair->next = hooks->list;
 	hooks->list = pair;
@@ -1764,7 +2087,7 @@ camel_object_bag_add (CamelObjectBag *bag, const void *key, void *vo)
  * the ref to.
  **/
 void *
-camel_object_bag_get (CamelObjectBag *bag, const void *key)
+camel_object_bag_get(CamelObjectBag *bag, const void *key)
 {
 	CamelObject *o;
 
@@ -1808,6 +2131,38 @@ camel_object_bag_get (CamelObjectBag *bag, const void *key)
 	return o;
 }
 
+/**
+ * camel_object_bag_peek:
+ * @bag: 
+ * @key: 
+ * 
+ * Lookup the object @key in @bag, ignoring any reservations.  If it
+ * isn't committed, then it isn't considered.  This should only be
+ * used where reliable transactional-based state is not required.
+ * 
+ * Unlike other 'peek' operations, the object is still reffed if
+ * found.
+ *
+ * Return value: A referenced object, or NULL if @key is not
+ * present in the bag.
+ **/
+void *
+camel_object_bag_peek(CamelObjectBag *bag, const void *key)
+{
+	CamelObject *o;
+
+	E_LOCK(type_lock);
+
+	o = g_hash_table_lookup(bag->object_table, key);
+	if (o) {
+		/* we use the same lock as the refcount */
+		o->ref_count++;
+	}
+	E_UNLOCK(type_lock);
+
+	return o;
+}
+
 /**
  * camel_object_bag_reserve:
  * @bag: 
@@ -1827,7 +2182,7 @@ camel_object_bag_get (CamelObjectBag *bag, const void *key)
  * Return value: 
  **/
 void *
-camel_object_bag_reserve (CamelObjectBag *bag, const void *key)
+camel_object_bag_reserve(CamelObjectBag *bag, const void *key)
 {
 	CamelObject *o;
 
@@ -1888,7 +2243,7 @@ camel_object_bag_reserve (CamelObjectBag *bag, const void *key)
  * Abort a key reservation.
  **/
 void
-camel_object_bag_abort (CamelObjectBag *bag, const void *key)
+camel_object_bag_abort(CamelObjectBag *bag, const void *key)
 {
 	E_LOCK(type_lock);
 
@@ -1897,6 +2252,38 @@ camel_object_bag_abort (CamelObjectBag *bag, const void *key)
 	E_UNLOCK(type_lock);
 }
 
+/**
+ * camel_object_bag_rekey:
+ * @bag: 
+ * @o: 
+ * @newkey: 
+ * 
+ * Re-key an object, atomically.  The key for object @o is set to
+ * @newkey, in an atomic manner.
+ *
+ * It is an api (fatal) error if @o is not currently in the bag.
+ **/
+void
+camel_object_bag_rekey(CamelObjectBag *bag, void *o, const void *newkey)
+{
+	void *oldkey;
+
+	E_LOCK(type_lock);
+
+	if (g_hash_table_lookup_extended(bag->key_table, o, NULL, &oldkey)) {
+		g_hash_table_remove(bag->object_table, oldkey);
+		g_hash_table_remove(bag->key_table, o);
+		bag->free_key(oldkey);
+		oldkey = bag->copy_key(newkey);
+		g_hash_table_insert(bag->object_table, oldkey, o);
+		g_hash_table_insert(bag->key_table, o, oldkey);
+	} else {
+		abort();
+	}
+
+	E_UNLOCK(type_lock);
+}
+
 static void
 save_bag(void *key, CamelObject *o, GPtrArray *list)
 {
@@ -1908,7 +2295,7 @@ save_bag(void *key, CamelObject *o, GPtrArray *list)
 /* get a list of all objects in the bag, ref'd
    ignores any reserved keys */
 GPtrArray *
-camel_object_bag_list (CamelObjectBag *bag)
+camel_object_bag_list(CamelObjectBag *bag)
 {
 	GPtrArray *list;
 
@@ -1923,7 +2310,7 @@ camel_object_bag_list (CamelObjectBag *bag)
 
 /* if bag is NULL, remove all bags from object */
 static void
-camel_object_bag_remove_unlocked (CamelObjectBag *inbag, CamelObject *o, CamelHookList *hooks)
+camel_object_bag_remove_unlocked(CamelObjectBag *inbag, CamelObject *o, CamelHookList *hooks)
 {
 	CamelHookPair *pair, *parent;
 	void *oldkey;
@@ -1953,7 +2340,7 @@ camel_object_bag_remove_unlocked (CamelObjectBag *inbag, CamelObject *o, CamelHo
 }
 
 void
-camel_object_bag_remove (CamelObjectBag *inbag, void *vo)
+camel_object_bag_remove(CamelObjectBag *inbag, void *vo)
 {
 	CamelObject *o = vo;
 	CamelHookList *hooks;
