@@ -2,24 +2,25 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2001-2003 Ximian, Inc. (www.ximian.com)
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,9 +32,15 @@
 
 #include "camel-sasl-cram-md5.h"
 #include "camel-sasl-digest-md5.h"
+#include "camel-sasl-gssapi.h"
 #include "camel-sasl-kerberos4.h"
 #include "camel-sasl-login.h"
 #include "camel-sasl-plain.h"
+#include "camel-sasl-popb4smtp.h"
+#include "camel-sasl-ntlm.h"
+
+
+#define w(x)
 
 static CamelObjectClass *parent_class = NULL;
 
@@ -55,7 +62,8 @@ static void
 camel_sasl_finalize (CamelSasl *sasl)
 {
 	g_free (sasl->service_name);
-	camel_object_unref (CAMEL_OBJECT (sasl->service));
+	g_free (sasl->mech);
+	camel_object_unref (sasl->service);
 }
 
 CamelType
@@ -81,7 +89,7 @@ camel_sasl_get_type (void)
 static GByteArray *
 sasl_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 {
-	g_warning ("sasl_challenge: Using default implementation!");
+	w(g_warning ("sasl_challenge: Using default implementation!"));
 	return NULL;
 }
 
@@ -103,7 +111,7 @@ GByteArray *
 camel_sasl_challenge (CamelSasl *sasl, GByteArray *token, CamelException *ex)
 {
 	g_return_val_if_fail (CAMEL_IS_SASL (sasl), NULL);
-
+	
 	return CS_CLASS (sasl)->challenge (sasl, token, ex);
 }
 
@@ -124,24 +132,24 @@ camel_sasl_challenge_base64 (CamelSasl *sasl, const char *token, CamelException
 	GByteArray *token_binary, *ret_binary;
 	char *ret;
 	int len;
-
+	
 	g_return_val_if_fail (CAMEL_IS_SASL (sasl), NULL);
-
+	
 	if (token) {
 		token_binary = g_byte_array_new ();
 		len = strlen (token);
 		g_byte_array_append (token_binary, token, len);
-		token_binary->len = base64_decode_simple (token_binary->data, len);
+		token_binary->len = camel_base64_decode_simple (token_binary->data, len);
 	} else
 		token_binary = NULL;
-
+	
 	ret_binary = camel_sasl_challenge (sasl, token_binary, ex);
 	if (token_binary)
 		g_byte_array_free (token_binary, TRUE);
 	if (!ret_binary)
 		return NULL;
-
-	ret = base64_encode_simple (ret_binary->data, ret_binary->len);
+	
+	ret = camel_base64_encode_simple (ret_binary->data, ret_binary->len);
 	g_byte_array_free (ret_binary, TRUE);
 
 	return ret;
@@ -177,32 +185,41 @@ CamelSasl *
 camel_sasl_new (const char *service_name, const char *mechanism, CamelService *service)
 {
 	CamelSasl *sasl;
-
+	
 	g_return_val_if_fail (service_name != NULL, NULL);
 	g_return_val_if_fail (mechanism != NULL, NULL);
 	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
-
+	
 	/* We don't do ANONYMOUS here, because it's a little bit weird. */
-
+	
 	if (!strcmp (mechanism, "CRAM-MD5"))
-		sasl = (CamelSasl *)camel_object_new (CAMEL_SASL_CRAM_MD5_TYPE);
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_CRAM_MD5_TYPE);
 	else if (!strcmp (mechanism, "DIGEST-MD5"))
-		sasl = (CamelSasl *)camel_object_new (CAMEL_SASL_DIGEST_MD5_TYPE);
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_DIGEST_MD5_TYPE);
+#ifdef HAVE_KRB5
+	else if (!strcmp (mechanism, "GSSAPI"))
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_GSSAPI_TYPE);
+#endif
 #ifdef HAVE_KRB4
 	else if (!strcmp (mechanism, "KERBEROS_V4"))
-		sasl = (CamelSasl *)camel_object_new (CAMEL_SASL_KERBEROS4_TYPE);
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_KERBEROS4_TYPE);
 #endif
 	else if (!strcmp (mechanism, "PLAIN"))
-		sasl = (CamelSasl *)camel_object_new (CAMEL_SASL_PLAIN_TYPE);
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_PLAIN_TYPE);
 	else if (!strcmp (mechanism, "LOGIN"))
-		sasl = (CamelSasl *)camel_object_new (CAMEL_SASL_LOGIN_TYPE);
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_LOGIN_TYPE);
+	else if (!strcmp (mechanism, "POPB4SMTP"))
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_POPB4SMTP_TYPE);
+	else if (!strcmp (mechanism, "NTLM"))
+		sasl = (CamelSasl *) camel_object_new (CAMEL_SASL_NTLM_TYPE);
 	else
 		return NULL;
 
+	sasl->mech = g_strdup (mechanism);
 	sasl->service_name = g_strdup (service_name);
 	sasl->service = service;
-	camel_object_ref (CAMEL_OBJECT (service));
-
+	camel_object_ref (service);
+	
 	return sasl;
 }
 
@@ -217,12 +234,16 @@ GList *
 camel_sasl_authtype_list (gboolean include_plain)
 {
 	GList *types = NULL;
-
+	
 	types = g_list_prepend (types, &camel_sasl_cram_md5_authtype);
 	types = g_list_prepend (types, &camel_sasl_digest_md5_authtype);
+#ifdef HAVE_KRB5
+	types = g_list_prepend (types, &camel_sasl_gssapi_authtype);
+#endif
 #ifdef HAVE_KRB4
 	types = g_list_prepend (types, &camel_sasl_kerberos4_authtype);
 #endif
+	types = g_list_prepend (types, &camel_sasl_ntlm_authtype);
 	if (include_plain)
 		types = g_list_prepend (types, &camel_sasl_plain_authtype);
 	
@@ -243,6 +264,10 @@ camel_sasl_authtype (const char *mechanism)
 		return &camel_sasl_cram_md5_authtype;
 	else if (!strcmp (mechanism, "DIGEST-MD5"))
 		return &camel_sasl_digest_md5_authtype;
+#ifdef HAVE_KRB5
+	else if (!strcmp (mechanism, "GSSAPI"))
+		return &camel_sasl_gssapi_authtype;
+#endif
 #ifdef HAVE_KRB4
 	else if (!strcmp (mechanism, "KERBEROS_V4"))
 		return &camel_sasl_kerberos4_authtype;
@@ -251,6 +276,10 @@ camel_sasl_authtype (const char *mechanism)
 		return &camel_sasl_plain_authtype;
 	else if (!strcmp (mechanism, "LOGIN"))
 		return &camel_sasl_login_authtype;
+	else if (!strcmp(mechanism, "POPB4SMTP"))
+		return &camel_sasl_popb4smtp_authtype;
+	else if (!strcmp (mechanism, "NTLM"))
+		return &camel_sasl_ntlm_authtype;
 	else
 		return NULL;
 }
