@@ -34,6 +34,8 @@
 
 #include <camel/camel-private.h>
 
+#include <camel/camel-i18n.h>
+
 #include "camel-imap4-store.h"
 #include "camel-imap4-engine.h"
 #include "camel-imap4-folder.h"
@@ -41,7 +43,9 @@
 #include "camel-imap4-command.h"
 #include "camel-imap4-utils.h"
 #include "camel-imap4-summary.h"
+#include "camel-imap4-store-summary.h"
 
+#define d(x) x
 
 static void camel_imap4_store_class_init (CamelIMAP4StoreClass *klass);
 static void camel_imap4_store_init (CamelIMAP4Store *store, CamelIMAP4StoreClass *klass);
@@ -143,6 +147,7 @@ static void
 camel_imap4_store_init (CamelIMAP4Store *store, CamelIMAP4StoreClass *klass)
 {
 	store->engine = NULL;
+	store->summary = NULL;
 }
 
 static void
@@ -150,6 +155,11 @@ camel_imap4_store_finalize (CamelObject *object)
 {
 	CamelIMAP4Store *store = (CamelIMAP4Store *) object;
 	
+	if (store->summary) {
+		camel_store_summary_save ((CamelStoreSummary *) store->summary);
+		camel_object_unref (store->summary);
+	}
+	
 	if (store->engine)
 		camel_object_unref (store->engine);
 	
@@ -161,6 +171,7 @@ static void
 imap4_construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, CamelException *ex)
 {
 	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
+	char *buf;
 	
 	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
 	if (camel_exception_is_set (ex))
@@ -168,6 +179,20 @@ imap4_construct (CamelService *service, CamelSession *session, CamelProvider *pr
 	
 	store->storage_path = camel_session_get_storage_path (session, service, ex);
 	store->engine = camel_imap4_engine_new (service, imap4_reconnect);
+	
+	/* setup/load the summary */
+	buf = g_alloca (strlen (store->storage_path) + 32);
+	sprintf (buf, "%s/.summary", store->storage_path);
+	store->summary = camel_imap4_store_summary_new ();
+	camel_store_summary_set_filename ((CamelStoreSummary *) store->summary, buf);
+	
+	buf = camel_url_to_string (service->url, CAMEL_URL_HIDE_ALL);
+	url = camel_url_new (buf, NULL);
+	g_free (buf);
+	camel_store_summary_set_uri_base ((CamelStoreSummary *) store->summary, url);
+	camel_url_free (url);
+	
+	camel_store_summary_load ((CamelStoreSummary *) store->summary);
 }
 
 static char *
@@ -181,39 +206,33 @@ imap4_get_name (CamelService *service, gboolean brief)
 }
 
 enum {
-	USE_SSL_NEVER,
-	USE_SSL_ALWAYS,
-	USE_SSL_WHEN_POSSIBLE
+	MODE_CLEAR,
+	MODE_SSL,
+	MODE_TLS,
 };
 
 #define SSL_PORT_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_SSL2 | CAMEL_TCP_STREAM_SSL_ENABLE_SSL3)
 #define STARTTLS_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_TLS)
 
 static gboolean
-connect_to_server (CamelIMAP4Engine *engine, struct hostent *host, int ssl_mode, int try_starttls, CamelException *ex)
+connect_to_server (CamelIMAP4Engine *engine, struct addrinfo *ai, int ssl_mode, CamelException *ex)
 {
 	CamelService *service = engine->service;
 	CamelStream *tcp_stream;
-	int port, ret;
+	CamelIMAP4Command *ic;
+	int id, ret;
 	
-	port = service->url->port ? service->url->port : 143;
-	
-	if (ssl_mode) {
+	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
-		if (try_starttls) {
-			tcp_stream = camel_tcp_stream_ssl_new (service->session, service->url->host, STARTTLS_FLAGS);
+		if (ssl_mode == MODE_TLS) {
+			tcp_stream = camel_tcp_stream_ssl_new_raw (service->session, service->url->host, STARTTLS_FLAGS);
 		} else {
-			port = service->url->port ? service->url->port : 993;
 			tcp_stream = camel_tcp_stream_ssl_new (service->session, service->url->host, SSL_PORT_FLAGS);
 		}
 #else
-		if (!try_starttls)
-			port = service->url->port ? service->url->port : 993;
-		
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				      _("Could not connect to %s (port %d): %s"),
-				      service->url->host, port,
-				      _("SSL unavailable"));
+				      _("Could not connect to %s: %s"),
+				      service->url->host, _("SSL unavailable"));
 		
 		return FALSE;
 #endif /* HAVE_SSL */
@@ -221,14 +240,14 @@ connect_to_server (CamelIMAP4Engine *engine, struct hostent *host, int ssl_mode,
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
 	
-	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, host, port)) == -1) {
+	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 					     _("Connection cancelled"));
 		else
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-					      _("Could not connect to %s (port %d): %s"),
-					      service->url->host, port,
+					      _("Could not connect to %s: %s"),
+					      service->url->host,
 					      g_strerror (errno));
 		
 		camel_object_unref (tcp_stream);
@@ -242,111 +261,106 @@ connect_to_server (CamelIMAP4Engine *engine, struct hostent *host, int ssl_mode,
 	if (camel_imap4_engine_capability (engine, ex) == -1)
 		return FALSE;
 	
-#ifdef HAVE_SSL
-	if (ssl_mode == USE_SSL_WHEN_POSSIBLE) {
-		/* try_starttls is always TRUE here */
-		if (engine->capa & CAMEL_IMAP4_CAPABILITY_STARTTLS)
-			goto starttls;
-	} else if (ssl_mode == USE_SSL_ALWAYS) {
-		if (try_starttls) {
-			if (engine->capa & CAMEL_IMAP4_CAPABILITY_STARTTLS) {
-				goto starttls;
-			} else {
-				/* server doesn't support STARTTLS, abort */
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-						      _("Failed to connect to IMAP server %s in secure mode: "
-							"Server does not support STARTTLS"),
-						      service->url->host);
-				return FALSE;
-			}
-		}
+	camel_imap4_store_summary_set_capabilities (((CamelIMAP4Store *) service)->summary, engine->capa);
+	
+	if (ssl_mode != MODE_TLS) {
+		/* we're done */
+		return TRUE;
 	}
-#endif /* HAVE_SSL */
 	
-	return TRUE;
-	
-#ifdef HAVE_SSL
- starttls:
-	
-	if (1) {
-		CamelIMAP4Command *ic;
-		int id;
+	if (!(engine->capa & CAMEL_IMAP4_CAPABILITY_STARTTLS)) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Failed to connect to IMAP server %s in secure mode: %s"),
+				      service->url->host, _("SSL negotiations failed"));
 		
-		ic = camel_imap4_engine_prequeue (engine, NULL, "STARTTLS\r\n");
-		while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
-			;
-		
-		if (id == -1 || ic->result != CAMEL_IMAP4_RESULT_OK) {
-			if (ic->result != CAMEL_IMAP4_RESULT_OK) {
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Failed to connect to IMAP server %s in secure mode: %s"),
-						      service->url->host, _("Unknown error"));
-			} else {
-				camel_exception_xfer (ex, &ic->ex);
-			}
-			
-			camel_imap4_command_unref (ic);
-			
-			return FALSE;
+		return FALSE;
+	}
+	
+	ic = camel_imap4_engine_prequeue (engine, NULL, "STARTTLS\r\n");
+	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
+		;
+	
+	if (id == -1 || ic->result != CAMEL_IMAP4_RESULT_OK) {
+		if (ic->result != CAMEL_IMAP4_RESULT_OK) {
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("Failed to connect to IMAP server %s in secure mode: %s"),
+					      service->url->host, _("Unknown error"));
+		} else {
+			camel_exception_xfer (ex, &ic->ex);
 		}
 		
 		camel_imap4_command_unref (ic);
+		
+		return FALSE;
 	}
 	
+	camel_imap4_command_unref (ic);
+	
 	return TRUE;
-#endif /* HAVE_SSL */
 }
 
 static struct {
 	char *value;
+	char *serv;
+	char *port;
 	int mode;
 } ssl_options[] = {
-	{ "",              USE_SSL_ALWAYS        },
-	{ "always",        USE_SSL_ALWAYS        },
-	{ "when-possible", USE_SSL_WHEN_POSSIBLE },
-	{ "never",         USE_SSL_NEVER         },
-	{ NULL,            USE_SSL_NEVER         },
+	{ "",              "imaps", "993", MODE_SSL   },  /* really old (1.x) */
+	{ "always",        "imaps", "993", MODE_SSL   },
+	{ "when-possible", "imap",  "143", MODE_TLS   },
+	{ "never",         "imap",  "143", MODE_CLEAR },
+	{ NULL,            "imap",  "143", MODE_CLEAR },
 };
 
 static gboolean
 connect_to_server_wrapper (CamelIMAP4Engine *engine, CamelException *ex)
 {
 	CamelService *service = engine->service;
-	const char *use_ssl;
-	struct hostent *h;
-	int ssl_mode;
-	int ret, i;
+	struct addrinfo *ai, hints;
+	const char *ssl_mode;
+	int mode, ret, i;
+	const char *port;
+	char *serv;
 	
-	if (!(h = camel_service_gethost (service, ex)))
+	if ((ssl_mode = camel_url_get_param (service->url, "use_ssl"))) {
+		for (i = 0; ssl_options[i].value; i++)
+			if (!strcmp (ssl_options[i].value, ssl_mode))
+				break;
+		mode = ssl_options[i].mode;
+		serv = ssl_options[i].serv;
+		port = ssl_options[i].port;
+	} else {
+		mode = MODE_CLEAR;
+		serv = "imap";
+		port = "143";
+	}
+	
+	if (service->url->port) {
+		serv = g_alloca (16);
+		sprintf (serv, "%d", service->url->port);
+		port = NULL;
+	}
+	
+	memset (&hints, 0, sizeof (hints));
+	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_family = PF_UNSPEC;
+	ai = camel_getaddrinfo (service->url->host, serv, &hints, ex);
+	if (ai == NULL && port != NULL && camel_exception_get_id(ex) != CAMEL_EXCEPTION_USER_CANCEL) {
+		camel_exception_clear (ex);
+		ai = camel_getaddrinfo (service->url->host, port, &hints, ex);
+	}
+	if (ai == NULL)
 		return FALSE;
 	
-	if ((use_ssl = camel_url_get_param (service->url, "use_ssl"))) {
-		for (i = 0; ssl_options[i].value; i++)
-			if (!strcmp (ssl_options[i].value, use_ssl))
-				break;
-		ssl_mode = ssl_options[i].mode;
-	} else {
-		ssl_mode = USE_SSL_NEVER;
+	if (!(ret = connect_to_server (engine, ai, mode, ex)) && mode == MODE_SSL) {
+		camel_exception_clear (ex);
+		ret = connect_to_server (engine, ai, MODE_TLS, ex);
+	} else if (!ret && mode == MODE_TLS) {
+		camel_exception_clear (ex);
+		ret = connect_to_server (engine, ai, MODE_CLEAR, ex);
 	}
 	
-	if (ssl_mode == USE_SSL_ALWAYS) {
-		/* First try the ssl port */
-		if (!(ret = connect_to_server (engine, h, ssl_mode, FALSE, ex))) {
-			if (camel_exception_get_id (ex) == CAMEL_EXCEPTION_SERVICE_UNAVAILABLE) {
-				/* The ssl port seems to be unavailable, lets try STARTTLS */
-				camel_exception_clear (ex);
-				ret = connect_to_server (engine, h, ssl_mode, TRUE, ex);
-			}
-		}
-	} else if (ssl_mode == USE_SSL_WHEN_POSSIBLE) {
-		/* If the server supports STARTTLS, use it */
-		ret = connect_to_server (engine, h, ssl_mode, TRUE, ex);
-	} else {
-		/* User doesn't care about SSL */
-		ret = connect_to_server (engine, h, USE_SSL_NEVER, FALSE, ex);
-	}
-	
-	camel_free_host (h);
+	camel_freeaddrinfo (ai);
 	
 	return ret;
 }
@@ -374,7 +388,7 @@ sasl_auth (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, const unsigned char
 	if (!(challenge = camel_sasl_challenge_base64 (sasl, (const char *) linebuf, ex)))
 		return -1;
 	
-	fprintf (stderr, "sending : %s\r\n", challenge);
+	d(fprintf (stderr, "sending : %s\r\n", challenge));
 	
 	if (camel_stream_printf (engine->ostream, "%s\r\n", challenge) == -1) {
 		g_free (challenge);
@@ -422,7 +436,7 @@ imap4_try_authenticate (CamelIMAP4Engine *engine, gboolean reprompt, const char
 		CamelServiceAuthType *mech;
 		
 		mech = g_hash_table_lookup (engine->authtypes, service->url->authmech);
-		sasl = camel_sasl_new ("imap4", mech->authproto, service);
+		sasl = camel_sasl_new ("imap", mech->authproto, service);
 		
 		ic = camel_imap4_engine_prequeue (engine, NULL, "AUTHENTICATE %s\r\n", service->url->authmech);
 		ic->plus = sasl_auth;
@@ -486,6 +500,8 @@ imap4_reconnect (CamelIMAP4Engine *engine, CamelException *ex)
 		g_free (errmsg);
 		errmsg = g_strdup (lex.desc);
 		camel_exception_clear (&lex);
+		g_free (service->url->passwd);
+		service->url->passwd = NULL;
 		reprompt = TRUE;
 	}
 	g_free (errmsg);
@@ -498,16 +514,25 @@ imap4_reconnect (CamelIMAP4Engine *engine, CamelException *ex)
 	if (camel_imap4_engine_namespace (engine, ex) == -1)
 		return FALSE;
 	
+	camel_imap4_store_summary_set_namespaces (((CamelIMAP4Store *) service)->summary, &engine->namespaces);
+	
 	return TRUE;
 }
 
 static gboolean
 imap4_connect (CamelService *service, CamelException *ex)
 {
+	CamelIMAP4Store *store = (CamelIMAP4Store *) service;
 	gboolean retval;
 	
+	if (!camel_session_is_online (service->session))
+		return TRUE;
+	
 	CAMEL_SERVICE_LOCK (service, connect_lock);
-	retval = imap4_reconnect (((CamelIMAP4Store *) service)->engine, ex);
+	if (store->engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED)
+		retval = imap4_reconnect (store->engine, ex);
+	else
+		retval = TRUE;
 	CAMEL_SERVICE_UNLOCK (service, connect_lock);
 	
 	return retval;
@@ -520,13 +545,18 @@ imap4_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 	CamelIMAP4Command *ic;
 	int id;
 	
-	if (clean && !store->engine->istream->disconnected) {
+	if (!camel_session_is_online (service->session))
+		return TRUE;
+	
+	CAMEL_SERVICE_LOCK (store, connect_lock);
+	if (clean && store->engine->state != CAMEL_IMAP4_ENGINE_DISCONNECTED) {
 		ic = camel_imap4_engine_queue (store->engine, NULL, "LOGOUT\r\n");
 		while ((id = camel_imap4_engine_iterate (store->engine)) < ic->id && id != -1)
 			;
 		
 		camel_imap4_command_unref (ic);
 	}
+	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 	
 	return 0;
 }
@@ -541,6 +571,9 @@ imap4_query_auth_types (CamelService *service, CamelException *ex)
 	GList *sasl_types, *t, *next;
 	gboolean connected;
 	
+	if (!camel_session_is_online (service->session))
+		return NULL;
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	connected = connect_to_server_wrapper (store->engine, ex);
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
@@ -561,83 +594,30 @@ imap4_query_auth_types (CamelService *service, CamelException *ex)
 	return g_list_prepend (sasl_types, &camel_imap4_password_authtype);
 }
 
-
-static char
-imap4_get_path_delim (CamelIMAP4Engine *engine, const char *full_name)
-{
-	/* FIXME: move this to utils so imap4-folder.c can share */
-	CamelIMAP4Namespace *namespace;
-	const char *slash;
-	size_t len;
-	char *top;
-	
-	if ((slash = strchr (full_name, '/')))
-		len = (slash - full_name);
-	else
-		len = strlen (full_name);
-	
-	top = g_alloca (len + 1);
-	memcpy (top, full_name, len);
-	top[len] = '\0';
-	
-	if (!g_ascii_strcasecmp (top, "INBOX"))
-		top = "INBOX";
-	
- retry:
-	namespace = engine->namespaces.personal;
-	while (namespace != NULL) {
-		if (!strcmp (namespace->path, top))
-			return namespace->sep;
-		namespace = namespace->next;
-	}
-	
-	namespace = engine->namespaces.other;
-	while (namespace != NULL) {
-		if (!strcmp (namespace->path, top))
-			return namespace->sep;
-		namespace = namespace->next;
-	}
-	
-	namespace = engine->namespaces.shared;
-	while (namespace != NULL) {
-		if (!strcmp (namespace->path, top))
-			return namespace->sep;
-		namespace = namespace->next;
-	}
-	
-	if (top[0] != '\0') {
-		/* look for a default namespace? */
-		top[0] = '\0';
-		goto retry;
-	}
-	
-	return '/';
-}
-
 static char *
 imap4_folder_utf7_name (CamelStore *store, const char *folder_name, char wildcard)
 {
 	char *real_name, *p;
-	char sep;
+	char sep = '\0';
 	int len;
 	
-	sep = imap4_get_path_delim (((CamelIMAP4Store *) store)->engine, folder_name);
-	
-	if (sep != '/') {
-		p = real_name = g_alloca (strlen (folder_name) + 1);
-		strcpy (real_name, folder_name);
-		while (*p != '\0') {
-			if (*p == '/')
-				*p = sep;
-			p++;
+	if (*folder_name) {
+		sep = camel_imap4_get_path_delim (((CamelIMAP4Store *) store)->summary, folder_name);
+		
+		if (sep != '/') {
+			p = real_name = g_alloca (strlen (folder_name) + 1);
+			strcpy (real_name, folder_name);
+			while (*p != '\0') {
+				if (*p == '/')
+					*p = sep;
+				p++;
+			}
+			
+			folder_name = real_name;
 		}
 		
-		folder_name = real_name;
-	}
-	
-	if (*folder_name)
 		real_name = camel_utf8_utf7 (folder_name);
-	else
+	} else
 		real_name = g_strdup ("");
 	
 	if (wildcard) {
@@ -658,6 +638,7 @@ static CamelFolder *
 imap4_get_folder (CamelStore *store, const char *folder_name, guint32 flags, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelFolder *folder = NULL;
 	camel_imap4_list_t *list;
 	CamelIMAP4Command *ic;
@@ -669,6 +650,18 @@ imap4_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Cam
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
+	if (!camel_session_is_online (session)) {
+		if ((flags & CAMEL_STORE_FOLDER_CREATE) != 0) {
+			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot create IMAP folders in offline mode."));
+		} else {
+			folder = camel_imap4_folder_new (store, folder_name, ex);
+		}
+		
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+		
+		return folder;
+	}
+	
 	/* make sure the folder exists - try LISTing it? */
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
 	ic = camel_imap4_engine_queue (engine, NULL, "LIST \"\" %S\r\n", utf7_name);
@@ -746,6 +739,7 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 	 * contain subfolders - delete them and re-create with the
 	 * proper hint */
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelFolderInfo *fi = NULL;
 	CamelIMAP4Command *ic;
 	char *utf7_name;
@@ -755,7 +749,7 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 	char sep;
 	int id;
 	
-	sep = imap4_get_path_delim (engine, parent_name);
+	sep = camel_imap4_get_path_delim (((CamelIMAP4Store *) store)->summary, parent_name);
 	
 	c = folder_name;
 	while (*c != '\0') {
@@ -770,6 +764,11 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 		c++;
 	}
 	
+	if (!camel_session_is_online (session)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot create IMAP folders in offline mode."));
+		return NULL;
+	}
+	
 	if (parent_name != NULL && *parent_name)
 		name = g_strdup_printf ("%s/%s", parent_name, folder_name);
 	else
@@ -807,6 +806,8 @@ imap4_create_folder (CamelStore *store, const char *parent_name, const char *fol
 		fi->unread = -1;
 		fi->total = -1;
 		
+		camel_imap4_store_summary_note_info (((CamelIMAP4Store *) store)->summary, fi);
+		
 		camel_object_trigger_event (store, "folder_created", fi);
 		break;
 	case CAMEL_IMAP4_RESULT_NO:
@@ -839,6 +840,7 @@ static void
 imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelFolder *selected = (CamelFolder *) engine->folder;
 	CamelIMAP4Command *ic, *ic0 = NULL;
 	CamelFolderInfo *fi;
@@ -855,6 +857,11 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 		return;
 	}
 	
+	if (!camel_session_is_online (session)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot delete IMAP folders in offline mode."));
+		return;
+	}
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
 	if (selected && !strcmp (folder_name, selected->full_name))
@@ -901,6 +908,8 @@ imap4_delete_folder (CamelStore *store, const char *folder_name, CamelException
 		fi->unread = -1;
 		fi->total = -1;
 		
+		camel_imap4_store_summary_unnote_info (((CamelIMAP4Store *) store)->summary, fi);
+		
 		camel_object_trigger_event (store, "folder_deleted", fi);
 		
 		camel_folder_info_free (fi);
@@ -927,6 +936,7 @@ static void
 imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_name, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	char *old_uname, *new_uname;
 	CamelIMAP4Command *ic;
 	int id;
@@ -939,6 +949,11 @@ imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_na
 		return;
 	}
 	
+	if (!camel_session_is_online (session)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename IMAP folders in offline mode."));
+		return;
+	}
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
 	old_uname = imap4_folder_utf7_name (store, old_name, '\0');
@@ -961,6 +976,7 @@ imap4_rename_folder (CamelStore *store, const char *old_name, const char *new_na
 	switch (ic->result) {
 	case CAMEL_IMAP4_RESULT_OK:
 		/* FIXME: need to update state on the renamed folder object */
+		/* FIXME: need to update cached summary info too */
 		break;
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
@@ -1074,6 +1090,11 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 	
 	url = camel_url_copy (engine->url);
 	
+	if (!strcmp (top, "") && (flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE)) {
+		/* clear the folder-info cache */
+		camel_store_summary_clear ((CamelStoreSummary *) ((CamelIMAP4Store *) store)->summary);
+	}
+	
 	for (i = 0; i < array->len; i++) {
 		list = array->pdata[i];
 		fi = g_malloc0 (sizeof (CamelFolderInfo));
@@ -1094,34 +1115,20 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 		fi->flags = list->flags;
 		fi->unread = -1;
 		fi->total = -1;
-		
-		if (!(flags & CAMEL_STORE_FOLDER_INFO_FAST)) {
-			if (folder && !strcmp (folder->full_name, fi->full_name)) {
-				/* can't STATUS this folder since it is SELECTED, besides - it would be wasteful */
-				CamelMessageInfo *info;
-				int index;
-				
-				fi->total = camel_folder_summary_count (folder->summary);
-				
-				fi->unread = 0;
-				for (index = 0; index < fi->total; index++) {
-					if (!(info = camel_folder_summary_index (folder->summary, index)))
-						continue;
-					
-					if ((info->flags & CAMEL_MESSAGE_SEEN) == 0)
-						fi->unread++;
-					
-					camel_folder_summary_info_free (folder->summary, info);
-				}
-			} else {
-				imap4_status (store, fi);
-			}
+
+		/* SELECTED folder, just get it from the folder */
+		if (folder && !strcmp (folder->full_name, fi->full_name)) {
+			camel_object_get(folder, NULL, CAMEL_FOLDER_TOTAL, &fi->total, CAMEL_FOLDER_UNREAD, &fi->unread, 0);
+		} else if (!(flags & CAMEL_STORE_FOLDER_INFO_FAST)) {
+			imap4_status (store, fi);
 		}
 		
 		g_free (list->name);
 		g_free (list);
 		
 		array->pdata[i] = fi;
+		
+		camel_imap4_store_summary_note_info (((CamelIMAP4Store *) store)->summary, fi);
 	}
 	
 	fi = camel_folder_info_build (array, top, '/', TRUE);
@@ -1130,6 +1137,8 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 	
 	g_ptr_array_free (array, TRUE);
 	
+	camel_store_summary_save ((CamelStoreSummary *) ((CamelIMAP4Store *) store)->summary);
+	
 	return fi;
 }
 
@@ -1137,6 +1146,7 @@ static CamelFolderInfo *
 imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelIMAP4Command *ic, *ic0 = NULL;
 	CamelFolderInfo *fi = NULL;
 	camel_imap4_list_t *list;
@@ -1146,23 +1156,30 @@ imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelE
 	char wildcard;
 	int id, i;
 	
+	if (top == NULL)
+		top = "";
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
-	if (engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED) {
-		if (!camel_service_connect ((CamelService *) store, ex))
-			return NULL;
-		
-		engine = ((CamelIMAP4Store *) store)->engine;
+	if (!camel_session_is_online (session) || engine->state == CAMEL_IMAP4_ENGINE_DISCONNECTED) {
+		fi = camel_imap4_store_summary_get_folder_info (((CamelIMAP4Store *) store)->summary, top, flags);
+		if (fi == NULL && camel_session_is_online (session)) {
+			/* folder info hasn't yet been cached and the store hasn't been
+			 * connected yet, but the network is available so we can connect
+			 * and query the server. */
+			goto check_online;
+		}
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+		return fi;
 	}
 	
+ check_online:
+	
 	if (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED)
 		cmd = "LSUB";
 	else
 		cmd = "LIST";
 	
-	if (top == NULL)
-		top = "";
-	
 	wildcard = (flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE) ? '*' : '%';
 	pattern = imap4_folder_utf7_name (store, top, wildcard);
 	array = g_ptr_array_new ();
@@ -1250,6 +1267,7 @@ static void
 imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelIMAP4Command *ic;
 	CamelFolderInfo *fi;
 	char *utf7_name;
@@ -1257,6 +1275,11 @@ imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcepti
 	const char *p;
 	int id;
 	
+	if (!camel_session_is_online (session)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot subscribe to IMAP folders in offline mode."));
+		return;
+	}
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
@@ -1290,6 +1313,8 @@ imap4_subscribe_folder (CamelStore *store, const char *folder_name, CamelExcepti
 		fi->unread = -1;
 		fi->total = -1;
 		
+		camel_imap4_store_summary_note_info (((CamelIMAP4Store *) store)->summary, fi);
+		
 		camel_object_trigger_event (store, "folder_subscribed", fi);
 		camel_folder_info_free (fi);
 		break;
@@ -1315,6 +1340,7 @@ static void
 imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelIMAP4Command *ic;
 	CamelFolderInfo *fi;
 	char *utf7_name;
@@ -1322,6 +1348,11 @@ imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExcep
 	const char *p;
 	int id;
 	
+	if (!camel_session_is_online (session)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot unsubscribe from IMAP folders in offline mode."));
+		return;
+	}
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
 	utf7_name = imap4_folder_utf7_name (store, folder_name, '\0');
@@ -1355,6 +1386,8 @@ imap4_unsubscribe_folder (CamelStore *store, const char *folder_name, CamelExcep
 		fi->unread = -1;
 		fi->total = -1;
 		
+		camel_imap4_store_summary_unnote_info (((CamelIMAP4Store *) store)->summary, fi);
+		
 		camel_object_trigger_event (store, "folder_unsubscribed", fi);
 		camel_folder_info_free (fi);
 		break;
@@ -1380,10 +1413,14 @@ static void
 imap4_noop (CamelStore *store, CamelException *ex)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	CamelSession *session = ((CamelService *) store)->session;
 	CamelFolder *folder = (CamelFolder *) engine->folder;
 	CamelIMAP4Command *ic;
 	int id;
 	
+	if (!camel_session_is_online (session))
+		return;
+	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
 	if (folder) {
