@@ -35,6 +35,7 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtktooltips.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libgnome/gnome-i18n.h>
 
 #include "Composer.h"
 
@@ -51,6 +52,8 @@
 
 #include <gal/e-text/e-entry.h>
 
+#include "widgets/misc/e-error.h"
+
 #include <camel/camel.h>
 #include "e-msg-composer-hdrs.h"
 #include "mail/mail-config.h"
@@ -266,7 +269,7 @@ static void
 account_removed_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs *hdrs)
 {
 	struct _EMsgComposerHdrsPrivate *priv = hdrs->priv;
-	GtkWidget *item, *omenu, *toplevel, *dialog;
+	GtkWidget *item, *omenu, *toplevel;
 	EAccount *acnt;
 	GSList *node;
 	
@@ -302,10 +305,8 @@ account_removed_cb (EAccountList *accounts, EAccount *account, EMsgComposerHdrs
 			toplevel = gtk_widget_get_toplevel ((GtkWidget *) hdrs);
 			gtk_widget_set_sensitive (toplevel, FALSE);
 			
-			dialog = gtk_message_dialog_new ((GtkWindow *) toplevel, GTK_DIALOG_MODAL |
-							 GTK_DIALOG_DESTROY_WITH_PARENT,
-							 GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "%s",
-							 _("You need to configure an account before you can compose mail."));
+			/* FIXME: this should offer a 'configure account' button, can we do that? */
+			e_error_run((GtkWindow *)toplevel, "mail-composer:all-accounts-deleted", NULL);
 		}
 	}
 }
@@ -510,7 +511,7 @@ post_browser_clicked_cb (GtkButton *button, EMsgComposerHdrs *hdrs)
 
 	dialog = em_folder_selector_new (emft, EM_FOLDER_SELECTOR_CAN_CREATE,
 	                                 _("Posting destination"),
-	                                 _("Choose folders to post the message to."));
+	                                 _("Choose folders to post the message to."), NULL);
 	
 	post_items = e_msg_composer_hdrs_get_post_to (hdrs);	
 	em_folder_selector_set_selected_list ((EMFolderSelector *) dialog, post_items);
@@ -535,7 +536,7 @@ header_new_recipient (EMsgComposerHdrs *hdrs, const char *name, const char *tip)
 	
 	priv = hdrs->priv;
 	
-	ret.label = gtk_button_new_with_label (name);
+	ret.label = gtk_button_new_with_mnemonic (name);
 	GTK_OBJECT_UNSET_FLAGS (ret.label, GTK_CAN_FOCUS);
 	g_signal_connect_data (ret.label, "clicked",
 			       G_CALLBACK (address_button_clicked_cb),
@@ -566,7 +567,7 @@ static void
 create_headers (EMsgComposerHdrs *hdrs)
 {
 	EMsgComposerHdrsPrivate *priv = hdrs->priv;
-	
+	AtkObject *a11y;	
 	/*
 	 * Reply-To:
 	 *
@@ -574,36 +575,39 @@ create_headers (EMsgComposerHdrs *hdrs)
 	 * because that causes from_changed to be called, which
 	 * expects the reply_to fields to be initialized.
 	 */
-	priv->reply_to.label = gtk_label_new (_("Reply-To:"));
+	priv->reply_to.label = gtk_label_new_with_mnemonic (_("_Reply-To:"));
 	priv->reply_to.entry = gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (priv->reply_to.label, priv->reply_to.entry);
 	
 	/*
 	 * From
 	 */
-	priv->from.label = gtk_label_new (_("From:"));
+	priv->from.label = gtk_label_new_with_mnemonic (_("Fr_om:"));
 	priv->from.entry = create_from_optionmenu (hdrs);
+	gtk_label_set_mnemonic_widget (priv->from.label, e_msg_composer_hdrs_get_from_omenu (hdrs));
 	
 	/*
 	 * Subject
 	 */
-	priv->subject.label = gtk_label_new (_("Subject:"));
+	priv->subject.label = gtk_label_new_with_mnemonic (_("S_ubject:"));
 	priv->subject.entry = gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (priv->subject.label, priv->subject.entry);
 	g_signal_connect (priv->subject.entry, "changed",
 			  G_CALLBACK (entry_changed), hdrs);
-	
+
 	/*
 	 * To, CC, and Bcc
 	 */
 	priv->to = header_new_recipient (
-		hdrs, _("To:"),
+		hdrs, _("_To:"),
 		_("Enter the recipients of the message"));
 	
 	priv->cc = header_new_recipient (
-		hdrs, _("Cc:"),
+		hdrs, _("_Cc:"),
 		_("Enter the addresses that will receive a carbon copy of the message"));
 	
 	priv->bcc = header_new_recipient (
-		hdrs, _("Bcc:"),
+		hdrs, _("_Bcc:"),
 		 _("Enter the addresses that will receive a carbon copy of "
 		   "the message without appearing in the recipient list of "
 		   "the message."));
@@ -611,7 +615,7 @@ create_headers (EMsgComposerHdrs *hdrs)
 	/*
 	 * Post-To
 	 */
-	priv->post_to.label = gtk_button_new_with_label (_("Post To:"));
+	priv->post_to.label = gtk_button_new_with_mnemonic (_("_Post To:"));
 	GTK_OBJECT_UNSET_FLAGS (priv->post_to.label, GTK_CAN_FOCUS);
 	g_signal_connect (priv->post_to.label, "clicked",
 			  G_CALLBACK (post_browser_clicked_cb), hdrs);
@@ -620,6 +624,10 @@ create_headers (EMsgComposerHdrs *hdrs)
 			      NULL);
 	
 	priv->post_to.entry = gtk_entry_new ();
+	a11y = gtk_widget_get_accessible (priv->post_to.entry);
+	if (a11y != NULL) {
+		atk_object_set_name (a11y, _("Post To:"));	
+	}
 	g_signal_connect(priv->post_to.entry, "changed",
 			 G_CALLBACK (post_entry_changed_cb), hdrs);
 }
