@@ -267,9 +267,11 @@ term_eval_and(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 			a1 = (char **)r1->value.ptrarray->pdata;
 			l1 = r1->value.ptrarray->len;
 			for (j=0;j<l1;j++) {
+			        gpointer ptr;
 				int n;
-				n = (int)g_hash_table_lookup(ht, a1[j]);
-				g_hash_table_insert(ht, a1[j], (void *)n+1);
+				ptr = g_hash_table_lookup(ht, a1[j]);
+				n = GPOINTER_TO_INT(ptr);
+				g_hash_table_insert(ht, a1[j], GINT_TO_POINTER(n+1));
 			}
 		} else if (r1->type == ESEXP_RES_BOOL) {
 			bool = bool && r1->value.bool;
@@ -901,7 +903,7 @@ parse_value(ESExp *f)
 		p(printf("got brace, its a list!\n"));
 		return parse_list(f, TRUE);
 	case G_TOKEN_STRING:
-		p(printf("got string\n"));
+		p(printf("got string '%s'\n", g_scanner_cur_value(gs).v_string));
 		t = parse_term_new(f, ESEXP_TERM_STRING);
 		t->value.string = g_strdup(g_scanner_cur_value(gs).v_string);
 		break;
@@ -920,7 +922,7 @@ parse_value(ESExp *f)
 		t->value.number = g_scanner_cur_value(gs).v_int;
 		if (negative)
 			t->value.number = -t->value.number;
-		p(printf("got int\n"));
+		p(printf("got int %d\n", t->value.number));
 		break;
 	case '#': {
 		char *str;
@@ -945,6 +947,7 @@ parse_value(ESExp *f)
 		break; }
 	case G_TOKEN_SYMBOL:
 		s = g_scanner_cur_value(gs).v_symbol;
+		p(printf("got symbol '%s'\n", s->name));
 		switch (s->type) {
 		case ESEXP_TERM_FUNC:
 		case ESEXP_TERM_IFUNC:
@@ -963,6 +966,7 @@ parse_value(ESExp *f)
 		}
 		break;
 	case G_TOKEN_IDENTIFIER:
+		p(printf("got unknown identifider '%s'\n", g_scanner_cur_value(gs).v_identifier));
 		e_sexp_fatal_error(f, "Unknown identifier: %s", g_scanner_cur_value(gs).v_identifier);
 		break;
 	default:
