@@ -28,6 +28,11 @@
 
 #include <gtk/gtkliststore.h>
 #include <gtk/gtktreeselection.h>
+#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtkcellrenderertoggle.h>
+#include <gtk/gtkcellrenderertext.h>
+
+#include <libgnome/gnome-i18n.h>
 
 #include "mail-component.h"
 #include "mail-config.h"
@@ -37,11 +42,12 @@
 #include "mail-send-recv.h"
 
 #include "e-util/e-account-list.h"
-
-#include "art/mark.xpm"
+#include "widgets/misc/e-error.h"
 
 #include "em-account-prefs.h"
 
+#include "em-account-editor.h"
+
 static void em_account_prefs_class_init (EMAccountPrefsClass *class);
 static void em_account_prefs_init       (EMAccountPrefs *prefs);
 static void em_account_prefs_finalise   (GObject *obj);
@@ -49,8 +55,6 @@ static void em_account_prefs_destroy    (GtkObject *object);
 
 static void mail_accounts_load (EMAccountPrefs *prefs);
 
-static GdkPixbuf *disabled_pixbuf = NULL;
-static GdkPixbuf *enabled_pixbuf = NULL;
 
 static GtkVBoxClass *parent_class = NULL;
 
@@ -91,10 +95,6 @@ em_account_prefs_class_init (EMAccountPrefsClass *klass)
 	gtk_object_class->destroy = em_account_prefs_destroy;
 	
 	object_class->finalize = em_account_prefs_finalise;
-	
-	/* setup static data */
-	disabled_pixbuf = NULL;
-	enabled_pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) mark_xpm);
 }
 
 static void
@@ -102,8 +102,6 @@ em_account_prefs_init (EMAccountPrefs *prefs)
 {
 	prefs->druid = NULL;
 	prefs->editor = NULL;
-	
-	gdk_pixbuf_render_pixmap_and_mask (enabled_pixbuf, &prefs->mark_pixmap, &prefs->mark_bitmap, 128);
 }
 
 static void
@@ -122,8 +120,6 @@ em_account_prefs_finalise (GObject *obj)
 	EMAccountPrefs *prefs = (EMAccountPrefs *) obj;
 	
 	g_object_unref (prefs->gui);
-	gdk_pixmap_unref (prefs->mark_pixmap);
-	g_object_unref (prefs->mark_bitmap);
 	
         G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -140,13 +136,17 @@ account_add_finished (EMAccountPrefs *prefs, GObject *deadbeef)
 	g_object_unref (prefs);
 }
 
+#include "em-config.h"
+
 static void
 account_add_clicked (GtkButton *button, gpointer user_data)
 {
 	EMAccountPrefs *prefs = (EMAccountPrefs *) user_data;
-	GtkWidget *parent;
 	
 	if (prefs->druid == NULL) {
+#if 0
+		GtkWidget *parent;
+
 		prefs->druid = (GtkWidget *) mail_config_druid_new ();
 		
 		parent = gtk_widget_get_toplevel ((GtkWidget *) prefs);
@@ -158,6 +158,12 @@ account_add_clicked (GtkButton *button, gpointer user_data)
 		
 		gtk_widget_show (prefs->druid);
 		g_object_ref (prefs);
+#else
+		EMAccountEditor *emae;
+
+		emae = em_account_editor_new(NULL, EMAE_DRUID);
+		gtk_widget_show(emae->editor);
+#endif
 	} else {
 		gdk_window_raise (prefs->druid->window);
 	}
@@ -190,8 +196,9 @@ account_edit_clicked (GtkButton *button, gpointer user_data)
 			gtk_tree_model_get (model, &iter, 3, &account, -1);
 		
 		if (account) {
+#if 0
 			GtkWidget *parent;
-			
+
 			parent = gtk_widget_get_toplevel ((GtkWidget *) prefs);
 			parent = GTK_WIDGET_TOPLEVEL (parent) ? parent : NULL;
 			
@@ -200,6 +207,15 @@ account_edit_clicked (GtkButton *button, gpointer user_data)
 			g_object_weak_ref ((GObject *) prefs->editor, (GWeakNotify) account_edit_finished, prefs);
 			gtk_widget_show (prefs->editor);
 			g_object_ref (prefs);
+#endif
+			/* test foo */
+			{
+				EMAccountEditor *emae;
+
+				emae = em_account_editor_new(account, EMAE_NOTEBOOK);
+				gtk_widget_show(emae->editor);
+			}
+
 		}
 	} else {
 		gdk_window_raise (prefs->editor->window);
@@ -214,7 +230,6 @@ account_delete_clicked (GtkButton *button, gpointer user_data)
 	EAccount *account = NULL;
 	EAccountList *accounts;
 	GtkTreeModel *model;
-	GtkWidget *confirm;
 	GtkTreeIter iter;
 	int ans;
 	
@@ -226,24 +241,7 @@ account_delete_clicked (GtkButton *button, gpointer user_data)
 	if (account == NULL || prefs->editor != NULL)
 		return;
 	
-	confirm = gtk_message_dialog_new (PREFS_WINDOW (prefs),
-					  GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-					  GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-					  _("Are you sure you want to delete this account?"));
-	
-	button = (GtkButton *) gtk_button_new_from_stock (GTK_STOCK_YES);
-	gtk_button_set_label (button, _("Delete"));
-	gtk_dialog_add_action_widget ((GtkDialog *) confirm, (GtkWidget *) button, GTK_RESPONSE_YES);
-	gtk_widget_show ((GtkWidget *) button);
-	
-	button = (GtkButton *) gtk_button_new_from_stock (GTK_STOCK_NO);
-	gtk_button_set_label (button, _("Don't delete"));
-	gtk_dialog_add_action_widget ((GtkDialog *) confirm, (GtkWidget *) button, GTK_RESPONSE_NO);
-	gtk_widget_show ((GtkWidget *) button);
-	
-	ans = gtk_dialog_run ((GtkDialog *) confirm);
-	gtk_widget_destroy (confirm);
-	
+	ans = e_error_run(PREFS_WINDOW(prefs), "mail:ask-delete-account", NULL);
 	if (ans == GTK_RESPONSE_YES) {
 		int len;
 		
@@ -257,8 +255,6 @@ account_delete_clicked (GtkButton *button, gpointer user_data)
 		
 		mail_config_write ();
 		
-		mail_autoreceive_setup ();
-		
 		gtk_list_store_remove ((GtkListStore *) model, &iter);
 		
 		len = e_list_length ((EList *) accounts);
@@ -296,12 +292,30 @@ account_default_clicked (GtkButton *button, gpointer user_data)
 }
 
 static void
-account_able_clicked (GtkButton *button, gpointer user_data)
+account_able_changed(EAccount *account)
 {
 	MailComponent *component = mail_component_peek ();
+
+	/* FIXME: do this directly by listening to the mail accounts changed events in the relevant components */
+
+	if (account->source->url) {
+		if (account->enabled)
+			mail_component_load_store_by_uri (component,
+							  account->source->url,
+							  account->name);
+		else
+			mail_component_remove_store_by_uri (component, account->source->url);
+	}
+
+	mail_config_write ();
+}
+
+static void
+account_able_clicked (GtkButton *button, gpointer user_data)
+{
 	EMAccountPrefs *prefs = user_data;
 	GtkTreeSelection *selection;
-	EAccount *account = NULL;
+	EAccount *account;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	
@@ -312,23 +326,10 @@ account_able_clicked (GtkButton *button, gpointer user_data)
 		gtk_list_store_set ((GtkListStore *) model, &iter, 0, account->enabled, -1);
 		
 		gtk_button_set_label (prefs->mail_able, account->enabled ? _("Disable") : _("Enable"));
-	}
-	
-	if (account) {
-		/* if the account got disabled, remove it from the
-		   folder-tree, otherwise add it to the folder-tree */
-		if (account->source->url) {
-			if (account->enabled)
-				mail_component_load_store_by_uri (component,
-								  account->source->url,
-								  account->name);
-			else
-				mail_component_remove_store_by_uri (component, account->source->url);
-		}
-		
-		mail_autoreceive_setup ();
-		
-		mail_config_write ();
+
+		/* let the rest of the application know it changed */
+		e_account_list_change(mail_config_get_accounts(), account);
+		account_able_changed(account);
 	}
 }
 
@@ -353,25 +354,13 @@ account_able_toggled (GtkCellRendererToggle *renderer, char *arg1, gpointer user
 		
 		if (gtk_tree_selection_iter_is_selected (selection, &iter))
 			gtk_button_set_label (prefs->mail_able, account->enabled ? _("Disable") : _("Enable"));
+
+		/* let the rest of the application know it changed */
+		e_account_list_change(mail_config_get_accounts(), account);
+		account_able_changed(account);
 	}
 	
 	gtk_tree_path_free (path);
-	
-	if (account) {
-		MailComponent *component = mail_component_peek ();
-		
-		/* if the account got disabled, remove it from the
-		   folder-tree, otherwise add it to the folder-tree */
-		if (account->source->url) {
-			if (account->enabled)
-				mail_component_load_store_by_uri (component, account->source->url, account->name);
-			else
-				mail_component_remove_store_by_uri (component, account->source->url);
-		}
-		
-		mail_autoreceive_setup ();
-		mail_config_write ();
-	}
 }
 
 static void
@@ -577,10 +566,3 @@ em_account_prefs_new (GNOME_Evolution_Shell shell)
 	
 	return (GtkWidget *) new;
 }
-
-
-void
-em_account_prefs_apply (EMAccountPrefs *prefs)
-{
-	/* nothing to do here... */
-}
