@@ -66,6 +66,7 @@
 
 #include "mail-config.h"
 #include "mail-signature-editor.h"
+#include "em-config.h"
 
 #define d(x)
 
@@ -389,8 +390,6 @@ sig_add_script_response (GtkWidget *widget, int button, EMComposerPrefs *prefs)
 					/* we're just editing an existing signature script */
 					g_free (sig->name);
 					sig->name = g_strdup (name);
-					g_free(sig->filename);
-					sig->filename = g_strdup(script);
 					e_signature_list_change (mail_config_get_signatures (), sig);
 				} else {
 					sig = mail_config_signature_new (script, TRUE, TRUE);
@@ -400,8 +399,6 @@ sig_add_script_response (GtkWidget *widget, int button, EMComposerPrefs *prefs)
 					g_object_unref (sig);
 				}
 				
-				mail_config_save_signatures();
-
 				gtk_widget_hide (prefs->sig_script_dialog);
 				g_strfreev (argv);
 				g_free (script);
@@ -844,6 +841,36 @@ toggle_button_init (EMComposerPrefs *prefs, GtkToggleButton *toggle, int not, co
 		gtk_widget_set_sensitive ((GtkWidget *) toggle, FALSE);
 }
 
+static GtkWidget *
+emcp_widget_glade(EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
+{
+	EMComposerPrefs *prefs = data;
+
+	return glade_xml_get_widget(prefs->gui, item->label);
+}
+
+/* plugin meta-data */
+static EMConfigItem emcp_items[] = {
+	{ E_CONFIG_BOOK, "", "composer_toplevel", emcp_widget_glade },
+	{ E_CONFIG_PAGE, "00.general", "vboxGeneral", emcp_widget_glade },
+	{ E_CONFIG_SECTION, "00.general/00.behavior", "vboxBehavior", emcp_widget_glade },
+	{ E_CONFIG_SECTION, "00.general/10.alerts", "vboxAlerts", emcp_widget_glade },
+	{ E_CONFIG_PAGE, "10.signatures", "vboxSignatures", emcp_widget_glade },
+	/* signature/signatures and signature/preview parts not usable */
+
+	{ E_CONFIG_PAGE, "20.spellcheck", "vboxSpellChecking", emcp_widget_glade },
+	{ E_CONFIG_SECTION, "20.spellcheck/00.languages", "vbox178", emcp_widget_glade },
+	{ E_CONFIG_SECTION, "20.spellcheck/00.options", "vboxOptions", emcp_widget_glade },
+};
+
+static void
+emcp_free(EConfig *ec, GSList *items, void *data)
+{
+	/* the prefs data is freed automagically */
+
+	g_slist_free(items);
+}
+
 static void
 em_composer_prefs_construct (EMComposerPrefs *prefs)
 {
@@ -854,22 +881,31 @@ em_composer_prefs_construct (EMComposerPrefs *prefs)
 	GtkTreeSelection *selection;
 	int style;
 	char *buf;
+	EMConfig *ec;
+	EMConfigTargetPrefs *target;
+	GSList *l;
+	int i;
 	
 	prefs->gconf = mail_config_get_gconf_client ();
 	
-	gui = glade_xml_new (EVOLUTION_GLADEDIR "/mail-config.glade", "composer_tab", NULL);
+	gui = glade_xml_new (EVOLUTION_GLADEDIR "/mail-config.glade", "composer_toplevel", NULL);
 	prefs->gui = gui;
 	prefs->sig_script_gui = glade_xml_new (EVOLUTION_GLADEDIR "/mail-config.glade", "vbox_add_script_signature", NULL);
-	
-	/* get our toplevel widget */
-	toplevel = glade_xml_get_widget (gui, "toplevel");
-	
-	/* reparent */
-	gtk_widget_ref (toplevel);
-	gtk_container_remove (GTK_CONTAINER (toplevel->parent), toplevel);
-	gtk_container_add (GTK_CONTAINER (prefs), toplevel);
-	gtk_widget_unref (toplevel);
-	
+
+	/** @HookPoint-EMConfig: Mail Composer Preferences
+	 * @Id: org.gnome.evolution.mail.composerPrefs
+	 * @Type: E_CONFIG_BOOK
+	 * @Class: org.gnome.evolution.mail.config:1.0
+	 * @Target: EMConfigTargetPrefs
+	 *
+	 * The mail composer preferences settings page.
+	 */
+	ec = em_config_new(E_CONFIG_BOOK, "org.gnome.evolution.mail.composerPrefs");
+	l = NULL;
+	for (i=0;i<sizeof(emcp_items)/sizeof(emcp_items[0]);i++)
+		l = g_slist_prepend(l, &emcp_items[i]);
+	e_config_add_items((EConfig *)ec, l, NULL, NULL, emcp_free, prefs);
+
 	/* General tab */
 	
 	/* Default Behavior */
@@ -992,8 +1028,13 @@ em_composer_prefs_construct (EMComposerPrefs *prefs)
 	g_signal_connect (prefs->sig_preview, "url_requested", G_CALLBACK (url_requested), NULL);
 	gtk_widget_show (GTK_WIDGET (prefs->sig_preview));
 	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (prefs->sig_preview));
-}
 
+	/* get our toplevel widget */
+	target = em_config_target_new_prefs(ec, prefs->gconf);
+	e_config_set_target((EConfig *)ec, (EConfigTarget *)target);
+	toplevel = e_config_create_widget((EConfig *)ec);
+	gtk_container_add (GTK_CONTAINER (prefs), toplevel);
+}
 
 GtkWidget *
 em_composer_prefs_new (void)
