@@ -41,9 +41,7 @@
 #include <gtkhtml/gtkhtml-stream.h>
 #include <gtkhtml/htmlengine.h>
 
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnome/gnome-i18n.h>
 
 #include <camel/camel-mime-message.h>
 #include <camel/camel-stream.h>
@@ -71,7 +69,7 @@
 #include "em-html-stream.h"
 #include "em-utils.h"
 
-#define d(x)
+#define d(x) 
 
 #define EFH_TABLE_OPEN "<table>"
 
@@ -455,6 +453,7 @@ static void emfh_gethttp(struct _EMFormatHTMLJob *job, int cancelled)
 	if (instream == NULL) {
 		char *proxy;
 
+		printf(" load http %d now=%d\n", job->format->load_http, job->format->load_http_now);
 
 		if (!(job->format->load_http_now
 		      || job->format->load_http == MAIL_CONFIG_HTTP_ALWAYS
@@ -660,9 +659,7 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 	struct _EMFormatHTMLCache *efhc;
 
 	camel_stream_printf (stream,
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=1 width=100%%><tr><td>\n"
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=0 width=100%%><tr><td>\n"
-			     "<table cellspacing=0 cellpadding=10><td><tr>\n",
+			     "<div style=\"border: solid #%06x 1px; background-color: #%06x; padding: 10px;\">\n",
 			     efh->frame_colour & 0xffffff, efh->content_colour & 0xffffff);
 
 	flags = efh->text_html_flags;
@@ -744,10 +741,7 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 	}
 
 	camel_object_unref(filtered_stream);
-	camel_stream_write_string(stream,
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n");
+	camel_stream_write_string(stream, "</div>\n");
 }
 
 static void
@@ -773,18 +767,13 @@ efh_text_enriched(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, E
 	camel_object_unref(enriched);
 
 	camel_stream_printf (stream,
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=1 width=100%%><tr><td>\n"
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=0 width=100%%><tr><td>\n"
-			     "<table cellspacing=0 cellpadding=10><td><tr>\n",
+			     "<div style=\"border: solid #%06x 1px; background-color: #%06x; padding: 10px;\">\n",
 			     efh->frame_colour & 0xffffff, efh->content_colour & 0xffffff);
 
 	em_format_format_text((EMFormat *)efh, (CamelStream *)filtered_stream, dw);
 	
 	camel_object_unref(filtered_stream);
-	camel_stream_write_string(stream,
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n");
+	camel_stream_write_string(stream, "</div>");
 }
 
 static void
@@ -801,8 +790,7 @@ efh_text_html(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFor
 	char *cid = NULL;
 
 	camel_stream_printf (stream,
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=1 width=100%%><tr><td>\n"
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=0 width=100%%><tr><td>\n"
+			     "<div style=\"border: solid #%06x 1px; background-color: #%06x;\">\n"
 			     "<!-- text/html -->\n",
 			     efh->frame_colour & 0xffffff, efh->content_colour & 0xffffff);
 
@@ -833,8 +821,7 @@ efh_text_html(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFor
 	d(printf("adding iframe, location %s\n", cid));
 	camel_stream_printf(stream,
 			    "<iframe src=\"%s\" frameborder=0 scrolling=no>could not get %s</iframe>\n"
-			    "</td></tr></table>\n"
-			    "</td></tr></table>\n",
+			    "</div>\n",
 			    cid, cid);
 	g_free(cid);
 }
@@ -938,9 +925,7 @@ efh_message_deliverystatus(EMFormatHTML *efh, CamelStream *stream, CamelMimePart
 
 	/* Yuck, this is copied from efh_text_plain */
 	camel_stream_printf (stream,
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=1 width=100%%><tr><td>\n"
-			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=0 width=100%%><tr><td>\n"
-			     "<table cellspacing=0 cellpadding=10><td><tr>\n",
+			     "<div style=\"border: solid #%06x 1px; background-color: #%06x; padding: 10px;\">\n",
 			     efh->frame_colour & 0xffffff, efh->content_colour & 0xffffff);
 
 	filtered_stream = camel_stream_filter_new_with_stream(stream);
@@ -953,10 +938,7 @@ efh_message_deliverystatus(EMFormatHTML *efh, CamelStream *stream, CamelMimePart
 	camel_stream_flush((CamelStream *)filtered_stream);
 	camel_stream_write_string(stream, "</tt>\n");
 
-	camel_stream_write_string(stream,
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n"
-				  "</td></tr></table>\n");
+	camel_stream_write_string(stream, "</div>");
 }
 
 static void
@@ -1183,55 +1165,43 @@ static void efh_format_do(struct _mail_msg *mm)
 		em_format_format_message((EMFormat *)m->format, (CamelStream *)m->estream, (CamelMedium *)m->message);
 	}
 
-	camel_stream_flush((CamelStream *)m->estream);
+	camel_stream_write_string((CamelStream *)m->estream, "</body>\n</html>\n");
+	camel_stream_close((CamelStream *)m->estream);
+	camel_object_unref(m->estream);
+	m->estream = NULL;
 
 	puri_level = ((EMFormat *)m->format)->pending_uri_level;
 	base = ((EMFormat *)m->format)->base;
 
-	do {
-		/* now dispatch any added tasks ... */
-		g_mutex_lock(m->format->priv->lock);
-		while ((job = (struct _EMFormatHTMLJob *)e_dlist_remhead(&m->format->priv->pending_jobs))) {
-			g_mutex_unlock(m->format->priv->lock);
-
-			/* This is an implicit check to see if the gtkhtml has been destroyed */
-			if (!cancelled)
-				cancelled = m->format->html == NULL;
-
-			/* Now do an explicit check for user cancellation */
-			if (!cancelled)
-				cancelled = camel_operation_cancel_check(NULL);
-
-			/* call jobs even if cancelled, so they can clean up resources */
-			((EMFormat *)m->format)->pending_uri_level = job->puri_level;
-			if (job->base)
-				((EMFormat *)m->format)->base = job->base;
-			job->callback(job, cancelled);
-			((EMFormat *)m->format)->base = base;
-
-			/* clean up the job */
-			camel_object_unref(job->stream);
-			if (job->base)
-				camel_url_free(job->base);
-			g_free(job);
-
-			g_mutex_lock(m->format->priv->lock);
-		}
+	/* now dispatch any added tasks ... */
+	g_mutex_lock(m->format->priv->lock);
+	while ((job = (struct _EMFormatHTMLJob *)e_dlist_remhead(&m->format->priv->pending_jobs))) {
 		g_mutex_unlock(m->format->priv->lock);
 
-		if (m->estream) {
-			/* Closing this base stream can queue more jobs, so we need
-			   to check the list again after we've finished */
-			d(printf("out of jobs, closing root stream\n"));
-			camel_stream_write_string((CamelStream *)m->estream, "</body>\n</html>\n");
-			camel_stream_close((CamelStream *)m->estream);
-			camel_object_unref(m->estream);
-			m->estream = NULL;
-		}
+		/* This is an implicit check to see if the gtkhtml has been destroyed */
+		if (!cancelled)
+			cancelled = m->format->html == NULL;
 
-		/* e_dlist_empty is atomic and doesn't need locking */
-	} while (!e_dlist_empty(&m->format->priv->pending_jobs));
+		/* Now do an explicit check for user cancellation */
+		if (!cancelled)
+			cancelled = camel_operation_cancel_check(NULL);
 
+		/* call jobs even if cancelled, so they can clean up resources */
+		((EMFormat *)m->format)->pending_uri_level = job->puri_level;
+		if (job->base)
+			((EMFormat *)m->format)->base = job->base;
+		job->callback(job, cancelled);
+		((EMFormat *)m->format)->base = base;
+
+		/* clean up the job */
+		camel_object_unref(job->stream);
+		if (job->base)
+			camel_url_free(job->base);
+		g_free(job);
+
+		g_mutex_lock(m->format->priv->lock);
+	}
+	g_mutex_unlock(m->format->priv->lock);
 	d(printf("out of jobs, done\n"));
 
 	((EMFormat *)m->format)->pending_uri_level = puri_level;
