@@ -27,7 +27,7 @@
 #include <stdio.h>
 #include <string.h>
 
-/* for the dialogue stuff */
+/* for the dialog stuff */
 #include <glib.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkdialog.h>
@@ -38,12 +38,13 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkbox.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <libgnome/gnome-i18n.h>
 
 #include "e-util/e-gtk-utils.h"
 #include "e-util/e-account-list.h"
 
 #include "widgets/misc/e-clipped-label.h"
-#include "filter/filter-filter.h"
+#include "em-filter-rule.h"
 #include "camel/camel-filter-driver.h"
 #include "camel/camel-folder.h"
 #include "camel/camel-operation.h"
@@ -56,6 +57,7 @@
 #include "mail-ops.h"
 #include "mail-send-recv.h"
 #include "mail-folder-cache.h"
+#include <e-util/e-icon-factory.h>
 
 #define d(x)
 
@@ -313,14 +315,21 @@ build_dialog (EAccountList *accounts, CamelFolder *outbox, const char *destinati
 	char *pretty_url;
 	EAccount *account;
 	EIterator *iter;
+	GList *icon_list;
 	
-	gd = (GtkDialog *)send_recv_dialog = gtk_dialog_new_with_buttons(_("Send & Receive Mail"), NULL, GTK_DIALOG_NO_SEPARATOR, NULL);
+	gd = (GtkDialog *)(send_recv_dialog = gtk_dialog_new_with_buttons(_("Send & Receive Mail"), NULL, GTK_DIALOG_NO_SEPARATOR, NULL));
 	gtk_window_set_modal ((GtkWindow *) gd, FALSE);
 	
 	stop = (GtkButton *)e_gtk_button_new_with_icon(_("Cancel _All"), GTK_STOCK_CANCEL);
 	gtk_widget_show((GtkWidget *)stop);
 	gtk_dialog_add_action_widget(gd, (GtkWidget *)stop, GTK_RESPONSE_CANCEL);
-	gnome_window_icon_set_from_file (GTK_WINDOW (gd), EVOLUTION_ICONSDIR "/send-receive.xpm");
+	
+	icon_list = e_icon_factory_get_icon_list ("stock_mail-send-receive");
+	if (icon_list) {
+		gtk_window_set_icon_list (GTK_WINDOW (gd), icon_list);
+		g_list_foreach (icon_list, (GFunc) g_object_unref, NULL);
+		g_list_free (icon_list);
+	}
 	
 	num_sources = 0;
 	
@@ -388,7 +397,7 @@ build_dialog (EAccountList *accounts, CamelFolder *outbox, const char *destinati
 		} else if (info->timeout_id == 0)
 			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 		
-		recv_icon = gtk_image_new_from_file (EVOLUTION_BUTTONSDIR "/receive-24.png");
+		recv_icon = e_icon_factory_get_image ("stock_mail-receive", E_ICON_SIZE_LARGE_TOOLBAR);
 		
 	       	pretty_url = format_url (source->url);
 		label = (GtkLabel *)gtk_label_new (pretty_url);
@@ -441,7 +450,7 @@ build_dialog (EAccountList *accounts, CamelFolder *outbox, const char *destinati
 		} else if (info->timeout_id == 0)
 			info->timeout_id = g_timeout_add (STATUS_TIMEOUT, operation_status_timeout, info);
 		
-		send_icon  = gtk_image_new_from_file (EVOLUTION_BUTTONSDIR "/send-24.png");
+		send_icon = e_icon_factory_get_image ("stock_mail-send", E_ICON_SIZE_LARGE_TOOLBAR);
 		
 		pretty_url = format_url (destination);
 		label = (GtkLabel *)gtk_label_new (pretty_url);
@@ -623,13 +632,6 @@ receive_done (char *uri, void *data)
 	free_send_info(NULL, info, NULL);
 }
 
-/* same for updating */
-static void
-receive_update_done(CamelStore *store, CamelFolderInfo *info, void *data)
-{
-	receive_done("", data);
-}
-
 /* although we dont do anythign smart here yet, there is no need for this interface to
    be available to anyone else.
    This can also be used to hook into which folders are being updated, and occasionally
@@ -687,7 +689,7 @@ receive_update_got_store (char *uri, CamelStore *store, void *data)
 	struct _send_info *info = data;
 	
 	if (store) {
-		mail_get_folderinfo (store, info->cancel, receive_update_got_folderinfo, info);
+		mail_note_store(store, info->cancel, receive_update_got_folderinfo, info);
 	} else {
 		receive_done ("", info);
 	}
@@ -755,8 +757,7 @@ GtkWidget *mail_send_receive (void)
 }
 
 struct _auto_data {
-	char *uri;
-	int keep;		/* keep on server flag */
+	EAccount *account;
 	int period;		/* in seconds */
 	int timeout_id;
 };
@@ -768,85 +769,100 @@ auto_timeout(void *data)
 {
 	struct _auto_data *info = data;
 
-	if (camel_session_is_online(session))
-		mail_receive_uri(info->uri, info->keep);
+	if (camel_session_is_online(session)) {
+		const char *uri = e_account_get_string(info->account, E_ACCOUNT_SOURCE_URL);
+		int keep = e_account_get_bool(info->account, E_ACCOUNT_SOURCE_KEEP_ON_SERVER);
+
+		mail_receive_uri(uri, keep);
+	}
 
 	return TRUE;
 }
 
-static void auto_setup_set(void *key, struct _auto_data *info, GHashTable *set)
+static void
+auto_account_removed(EAccountList *eal, EAccount *ea, void *dummy)
 {
-	g_hash_table_insert(set, info->uri, info);
+	struct _auto_data *info = g_object_get_data((GObject *)ea, "mail-autoreceive");
+
+	g_return_if_fail(info != NULL);
+
+	if (info->timeout_id) {
+		g_source_remove(info->timeout_id);
+		info->timeout_id = 0;
+	}
 }
 
-static void auto_clean_set(void *key, struct _auto_data *info, GHashTable *set)
+static void
+auto_account_finalised(struct _auto_data *info)
 {
-	d(printf("removing auto-check for %s %p\n", info->uri, info));
-	g_hash_table_remove(set, info->uri);
-	g_source_remove(info->timeout_id);
-	g_free(info->uri);
+	if (info->timeout_id)
+		g_source_remove(info->timeout_id);
 	g_free(info);
 }
 
+static void
+auto_account_commit(struct _auto_data *info)
+{
+	int period, check;
+
+	check = info->account->enabled
+		&& e_account_get_bool(info->account, E_ACCOUNT_SOURCE_AUTO_CHECK)
+		&& e_account_get_string(info->account, E_ACCOUNT_SOURCE_URL);
+	period = e_account_get_int(info->account, E_ACCOUNT_SOURCE_AUTO_CHECK_TIME)*60;
+	period = MAX(60, period);
+
+	if (info->timeout_id
+	    && (!check
+		|| period != info->period)) {
+		g_source_remove(info->timeout_id);
+		info->timeout_id = 0;
+	}
+	info->period = period;
+	if (check && info->timeout_id == 0)
+		info->timeout_id = g_timeout_add(info->period*1000, auto_timeout, info);
+}
+
+static void
+auto_account_added(EAccountList *eal, EAccount *ea, void *dummy)
+{
+	struct _auto_data *info;
+
+	info = g_malloc0(sizeof(*info));
+	info->account = ea;
+	g_object_set_data_full((GObject *)ea, "mail-autoreceive", info, (GDestroyNotify)auto_account_finalised);
+	auto_account_commit(info);
+}
+
+static void
+auto_account_changed(EAccountList *eal, EAccount *ea, void *dummy)
+{
+	struct _auto_data *info = g_object_get_data((GObject *)ea, "mail-autoreceive");
+
+	g_return_if_fail(info != NULL);
+
+	auto_account_commit(info);
+}
+
 /* call to setup initial, and after changes are made to the config */
 /* FIXME: Need a cleanup funciton for when object is deactivated */
 void
-mail_autoreceive_setup (void)
+mail_autoreceive_init(void)
 {
 	EAccountList *accounts;
-	GHashTable *set_hash;
 	EIterator *iter;
 
+	if (auto_active)
+		return;
+
 	accounts = mail_config_get_accounts ();
-	
-	if (auto_active == NULL)
-		auto_active = g_hash_table_new(g_str_hash, g_str_equal);
-	
-	set_hash = g_hash_table_new(g_str_hash, g_str_equal);
-	g_hash_table_foreach(auto_active, (GHFunc)auto_setup_set, set_hash);
-	
-	iter = e_list_get_iterator ((EList *) accounts);
-	while (e_iterator_is_valid (iter)) {
-		EAccountService *source;
-		EAccount *account;
-		
-		account = (EAccount *) e_iterator_get (iter);
-		source = account->source;
-		
-		if (account->enabled && source->url && source->auto_check) {
-			struct _auto_data *info;
-			
-			d(printf("setting up auto-receive mail for : %s %dm\n", source->url, source->auto_check_time));
-			
-			g_hash_table_remove(set_hash, source->url);
-			info = g_hash_table_lookup(auto_active, source->url);
-			if (info) {
-				info->keep = source->keep_on_server;
-				if (info->period != source->auto_check_time*60) {
-					info->period = source->auto_check_time*60;
-					g_source_remove(info->timeout_id);
-					info->timeout_id = g_timeout_add(info->period*1000, auto_timeout, info);
-				}
-			} else {
-				info = g_malloc0(sizeof(*info));
-				info->uri = g_strdup(source->url);
-				info->keep = source->keep_on_server;
-				info->period = source->auto_check_time*60;
-				info->timeout_id = g_timeout_add(info->period*1000, auto_timeout, info);
-				g_hash_table_insert(auto_active, info->uri, info);
-				/* If we do this at startup, it can cause the logon dialog to be hidden,
-				   so lets not */
-				/*mail_receive_uri(source->url, source->keep_on_server);*/
-			}
-		}
-		
-		e_iterator_next (iter);
-	}
-	
-	g_object_unref (iter);
-	
-	g_hash_table_foreach(set_hash, (GHFunc)auto_clean_set, auto_active);
-	g_hash_table_destroy(set_hash);
+	auto_active = g_hash_table_new(g_str_hash, g_str_equal);
+
+	g_signal_connect(accounts, "account-added", G_CALLBACK(auto_account_added), NULL);
+	g_signal_connect(accounts, "account-removed", G_CALLBACK(auto_account_removed), NULL);
+	g_signal_connect(accounts, "account-changed", G_CALLBACK(auto_account_changed), NULL);
+
+	for (iter = e_list_get_iterator((EList *)accounts);e_iterator_is_valid(iter);e_iterator_next(iter))
+		auto_account_added(accounts, (EAccount *)e_iterator_get(iter), NULL);
 }
 
 /* we setup the download info's in a hashtable, if we later need to build the gui, we insert
