@@ -108,6 +108,9 @@ struct _MessageList {
 	/* Current search string, or %NULL */
 	char *search;
 	
+	/* are we regenerating the message_list because set_folder was just called? */
+	guint just_set_folder : 1;
+	
 	/* Are we displaying threaded view? */
 	guint threaded : 1;
 	
@@ -119,7 +122,10 @@ struct _MessageList {
 	
 	/* is the message-list object in a destroyed state? */
 	guint destroyed : 1;
-	
+
+	/* frozen count */
+	guint frozen:16;
+
 	/* Where the ETree cursor is. */
 	int cursor_row;
 	char *cursor_uid;
@@ -133,6 +139,10 @@ struct _MessageList {
 	/* list of outstanding regeneration requests */
 	GList *regen;
 	char *pending_select_uid; /* set if we were busy regnerating while we had a select come in */
+	guint regen_timeout_id;
+	void *regen_timeout_msg;
+
+	char *frozen_search;	/* to save search took place while we were frozen */
 
 	/* the current camel folder thread tree, if any */
 	struct _CamelFolderThread *thread_tree;
@@ -147,6 +157,7 @@ typedef struct {
 	/* signals - select a message */
 	void (*message_selected) (MessageList *ml, const char *uid);
 	void (*message_list_built) (MessageList *ml);
+	void (*message_list_scrolled) (MessageList *ml);
 } MessageListClass;
 
 typedef void (*MessageListForeachFunc) (MessageList *message_list,
@@ -154,8 +165,10 @@ typedef void (*MessageListForeachFunc) (MessageList *message_list,
 					gpointer user_data);
 
 typedef enum {
-	MESSAGE_LIST_SELECT_NEXT = 1,
-	MESSAGE_LIST_SELECT_PREVIOUS = -1
+	MESSAGE_LIST_SELECT_NEXT = 0,
+	MESSAGE_LIST_SELECT_PREVIOUS = 1,
+	MESSAGE_LIST_SELECT_DIRECTION = 1, /* direction mask */
+	MESSAGE_LIST_SELECT_WRAP = 1<<1, /* option bit */
 } MessageListSelectDirection;
 
 GtkType        message_list_get_type   (void);
@@ -166,15 +179,19 @@ void           message_list_foreach    (MessageList *message_list,
 					MessageListForeachFunc callback,
 					gpointer user_data);
 
+void	       message_list_freeze(MessageList *ml);
+void	       message_list_thaw(MessageList *ml);
+
 GPtrArray     *message_list_get_selected(MessageList *ml);
+void           message_list_set_selected(MessageList *ml, GPtrArray *uids);
 void	       message_list_free_uids(MessageList *ml, GPtrArray *uids);
 
 /* select next/prev message helpers */
 gboolean       message_list_select     (MessageList *message_list,
 					MessageListSelectDirection direction,
 					guint32 flags,
-					guint32 mask,
-					gboolean wraparound);
+					guint32 mask);
+gboolean message_list_can_select(MessageList *ml, MessageListSelectDirection direction, guint32 flags, guint32 mask);
 
 void           message_list_select_uid (MessageList *message_list,
 					const char *uid);
@@ -188,7 +205,6 @@ void           message_list_invert_selection (MessageList *ml);
 
 /* clipboard stuff */
 void	       message_list_copy(MessageList *ml, gboolean cut);
-gboolean       message_list_has_primary_selection(MessageList *ml);
 void           message_list_paste (MessageList *ml);
 
 /* info */
