@@ -24,11 +24,15 @@
 #include <config.h>
 #endif
 
+#include <sys/types.h>
+#include <dirent.h>
+
 #include "e-shell.h"
 
 #include "e-util/e-dialog-utils.h"
 #include "e-util/e-bconf-map.h"
 #include "e-util/e-fsutils.h"
+#include "e-util/e-passwords.h"
 #include "widgets/misc/e-error.h"
 
 #include "e-shell-constants.h"
@@ -37,6 +41,7 @@
 #include "e-shell-startup-wizard.h"
 
 #include "e-shell-marshal.h"
+#include "es-event.h"
 
 #include "evolution-shell-component-utils.h"
 
@@ -151,7 +156,41 @@ static void
 set_interactive (EShell *shell,
 		 gboolean interactive)
 {
-	/* FIXME TODO */
+	GSList *component_list;
+	GSList *p;
+	GList *first_element;
+	int num_windows;
+	GtkWidget *view;
+
+	g_return_if_fail (E_IS_SHELL (shell));
+	
+	shell->priv->is_interactive = interactive;
+
+	num_windows = g_list_length (shell->priv->windows);
+	
+	/* We want to send the "interactive" message only when the first
+	window is created */
+	if (num_windows != 1)
+		return;
+
+	first_element = g_list_first (shell->priv->windows);
+	view = GTK_WIDGET (first_element->data);
+
+	component_list = e_component_registry_peek_list (shell->priv->component_registry);
+
+	for (p = component_list; p != NULL; p = p->next) {
+		EComponentInfo *info = p->data;
+		CORBA_Environment ev;
+
+		CORBA_exception_init (&ev);
+
+		GNOME_Evolution_Component_interactive (info->iface, interactive,GPOINTER_TO_INT (GDK_WINDOW_XWINDOW (view->window)), &ev);
+
+		/* Ignore errors, the components can decide to not implement
+		   this interface. */
+		
+		CORBA_exception_free (&ev);
+	}
 }
 
 
@@ -207,24 +246,33 @@ impl_Shell_handleURI (PortableServer_Servant servant,
 {
 	EShell *shell = E_SHELL (bonobo_object_from_servant (servant));
 	EComponentInfo *component_info;
-	const char *colon_p;
-	char *schema;
+	char *schema, *p;
+	int show = FALSE;
 
-	colon_p = strchr (uri, ':');
-	if (colon_p == NULL)
-		schema = g_strdup (uri);
-	else
-		schema = g_strndup (uri, colon_p - uri);
+	schema = g_alloca(strlen(uri)+1);
+	strcpy(schema, uri);
+	p = strchr(schema, ':');
+	if (p)
+		*p = 0;
 
-	component_info = e_component_registry_peek_info_for_uri_schema (shell->priv->component_registry, schema);
-	g_free (schema);
+	component_info = e_component_registry_peek_info(shell->priv->component_registry, ECR_FIELD_SCHEMA, schema);
+	if (component_info == NULL) {
+		show = TRUE;
+		component_info = e_component_registry_peek_info(shell->priv->component_registry, ECR_FIELD_ALIAS, schema);
+	}
 
 	if (component_info == NULL) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_GNOME_Evolution_Shell_UnsupportedSchema, NULL);
 		return;
 	}
 
+	if (show && shell->priv->windows)
+		e_shell_window_switch_to_component((EShellWindow *)shell->priv->windows->data, component_info->id);
+
 	GNOME_Evolution_Component_handleURI (component_info->iface, uri, ev);
+	/* not an error not to implement it */
+	if (ev->_id != NULL && strcmp(ev->_id, ex_CORBA_NO_IMPLEMENT) == 0)
+		memset(ev, 0, sizeof(*ev));
 }
 
 static void
@@ -241,6 +289,9 @@ impl_Shell_setLineStatus (PortableServer_Servant servant,
 	bonobo_object = bonobo_object_from_servant (servant);
 	shell = E_SHELL (bonobo_object);
 
+	/* let the password manager know out online status */
+	e_passwords_set_online(online);
+
 	if (online)
 		e_shell_go_online (shell, NULL);
 	else
@@ -304,29 +355,6 @@ window_weak_notify (void *data,
 	}
 }
 
-static EShellWindow *
-create_window (EShell *shell,
-	       const char *component_id,
-	       EShellWindow *template_window)
-{
-	EShellPrivate *priv;
-	EShellWindow *window;
-
-	priv = shell->priv;
-
-	window = E_SHELL_WINDOW (e_shell_window_new (shell, component_id));
-
-	g_signal_connect (window, "delete_event", G_CALLBACK (window_delete_event_cb), shell);
-	g_object_weak_ref (G_OBJECT (window), window_weak_notify, shell);
-
-	shell->priv->windows = g_list_prepend (shell->priv->windows, window);
-
-	g_signal_emit (shell, signals[NEW_WINDOW_CREATED], 0, window);
-
-	return window;
-}
-
-
 /* GObject methods.  */
 
 static void
@@ -521,44 +549,54 @@ detect_version (GConfClient *gconf, int *major, int *minor, int *revision)
 	g_free (evolution_dir);
 }
 
-static void
-attempt_upgrade (EShell *shell)
+/* calls components to perform upgrade */
+static gboolean
+attempt_upgrade (EShell *shell, int major, int minor, int revision)
 {
-	GConfClient *gconf_client;
-	int major = 0, minor = 0, revision = 0;
-	char *version_string;
+	GSList *component_infos, *p;
+	gboolean success;
+	int res;
 
-	gconf_client = gconf_client_get_default ();
+	success = TRUE;
 
-	detect_version (gconf_client, &major, &minor, &revision);
+	component_infos = e_component_registry_peek_list (shell->priv->component_registry);
+	for (p = component_infos; success && p != NULL; p = p->next) {
+		const EComponentInfo *info = p->data;
+		CORBA_Environment ev;
 
-	/* if upgrading from < 1.5, we need to copy most data from ~/evolution to ~/.evolution */
-	if (major == 1 && minor < 5) {
-		char *path;
-		long size, space;
+		CORBA_exception_init (&ev);
+		
+		GNOME_Evolution_Component_upgradeFromVersion (info->iface, major, minor, revision, &ev);
+		
+		if (BONOBO_EX (&ev)) {
+			char *exception_text;
+			CORBA_char *id = CORBA_exception_id(&ev);
 
-		path = g_build_filename(g_get_home_dir(), "evolution", NULL);
-		size = e_fsutils_usage(path);
-		g_free(path);
-		space = e_fsutils_avail(g_get_home_dir());
-		if (size != -1 && space != -1 && space < size) {
-			char *required = g_strdup_printf(_("%ld KB"), size);
-			char *have = g_strdup_printf(_("%ld KB"), space);
+			if (strcmp (id, ex_CORBA_NO_IMPLEMENT) == 0) {
+				/* Ignore components that do not implement this version, it
+				   might just mean that they don't need an upgrade path. */
+			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UpgradeFailed) == 0) {
+				GNOME_Evolution_Component_UpgradeFailed *ex = CORBA_exception_value(&ev);
 
-			e_error_run(NULL, "shell:upgrade-nospace", required, have, NULL);
-			g_free(required);
-			g_free(have);
-			_exit(0);
+				res = e_error_run(NULL, "shell:upgrade-failed", ex->what, ex->why, NULL);
+				if (res == GTK_RESPONSE_CANCEL)
+					success = FALSE;
+			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UnsupportedVersion) == 0) {
+				/* This is non-fatal */
+				/* DO WE CARE??? */
+				printf("Upgrade of component failed, unsupported prior version\n");
+			} else {
+				exception_text = bonobo_exception_get_text (&ev);
+				res = e_error_run(NULL, "shell:upgrade-failed", exception_text, _("Uknown system error."), NULL);
+				g_free (exception_text);
+				if (res == GTK_RESPONSE_CANCEL)
+					success = FALSE;
+			}
 		}
+		CORBA_exception_free (&ev);
 	}
 
-	if (!e_shell_attempt_upgrade (shell, major, minor, revision))
-		_exit(0);
-
-	version_string = g_strdup_printf ("%s.%s", BASE_VERSION, UPGRADE_REVISION);
-	gconf_client_set_string (gconf_client, "/apps/evolution/version", version_string, NULL);
-	g_object_unref (gconf_client);
-	g_free (version_string);
+	return success;
 }
 
 /**
@@ -601,27 +639,17 @@ e_shell_construct (EShell *shell,
 	while (gtk_events_pending ())
 		gtk_main_iteration ();	
 	
-	/* activate all the components */
+	/* activate all the components (peek list does this implictly) */
 	component = e_component_registry_peek_list (shell->priv->component_registry);
-	while (component != NULL) {
-		const EComponentInfo *info = component->data;
-		CORBA_Environment ev;
-		
-		CORBA_exception_init (&ev);
-		
-		e_component_registry_activate (shell->priv->component_registry, info->id, &ev);
-		
-		CORBA_exception_free (&ev);
-		
-		component = component->next;
-	}
 	
-	attempt_upgrade(shell);
+	e_shell_attempt_upgrade(shell);
 
+#if 0
 	if (e_shell_startup_wizard_create () == FALSE) {
 		bonobo_object_unref (BONOBO_OBJECT (shell));
 		exit (0);
 	}
+#endif
 
 	priv->is_initialized = TRUE;
 
@@ -640,6 +668,8 @@ e_shell_construct (EShell *shell,
 		g_assert_not_reached ();
 	}
 
+	e_passwords_set_online(start_online);
+
 	if (start_online)
 		e_shell_go_online (shell, NULL);
 
@@ -680,79 +710,161 @@ e_shell_new (EShellStartupLineMode startup_line_mode,
 	return new;
 }
 
+static int
+remove_dir(const char *root, const char *path)
+{
+	DIR *dir;
+	struct dirent *d;
+	int res = -1;
+	char *new = NULL;
+	struct stat st;
+
+	dir = opendir(path);
+	if (dir == NULL)
+		return -1;
+
+	while ( (d = readdir(dir)) ) {
+		if (!strcmp(d->d_name, ".")
+		    || !strcmp(d->d_name, ".."))
+			continue;
+
+		new = g_build_filename(path, d->d_name, NULL);
+		if (stat(new, &st) == -1)
+			goto fail;
+
+		/* make sure we're really removing something from evolution dir */
+		g_assert(strlen(path) >= strlen(root)
+			 && strncmp(root, path, strlen(root)) == 0);
+
+		if (S_ISDIR(st.st_mode)) {
+			if (remove_dir(root, new) == -1)
+				goto fail;
+		} else {
+			if (unlink(new) == -1)
+				goto fail;
+		}
+		g_free(new);
+		new = NULL;
+	}
+
+	res = rmdir(path);
+fail:
+	g_free(new);
+	closedir(dir);
+	return res;
+}
+
 /**
  * e_shell_attempt_upgrade:
  * @shell: 
- * @from_version: 
  * 
- * Upgrade config and components from the specified version name.
+ * Upgrade config and components from the currently installed version.
  * 
- * Return value: %TRUE If it works, %FALSE if it fails (it should only fail if
- * upgrade from @from_version is unsupported).
+ * Return value: %TRUE If it works.  If it fails the application will exit.
  **/
 gboolean
-e_shell_attempt_upgrade (EShell *shell, int major, int minor, int revision)
+e_shell_attempt_upgrade (EShell *shell)
 {
-	GSList *component_infos, *p;
-	int current_major, current_minor, current_revision;
-	gboolean success;
-	int res;
+	GConfClient *gconf_client;
+	int major = 0, minor = 0, revision = 0;
+	int lmajor, lminor, lrevision;
+	int cmajor, cminor, crevision;
+	char *version_string, *last_version = NULL;
+	int done_upgrade = FALSE;
+	char *oldpath;
+	struct stat st;
 
-	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
+	gconf_client = gconf_client_get_default();
 
-	sscanf (BASE_VERSION, "%u.%u", &current_major, &current_minor);
+	oldpath = g_build_filename(g_get_home_dir(), "evolution", NULL);
 
-	current_revision = atoi (UPGRADE_REVISION);
+	g_assert(sscanf(BASE_VERSION, "%u.%u", &cmajor, &cminor) == 2);
+	crevision = atoi(UPGRADE_REVISION);
 
-	if (! (current_major > major
-	       || (current_major == major && current_minor > minor)
-	       || (current_minor == current_minor && current_revision > revision))) {
-		return TRUE;
-	}
+	detect_version (gconf_client, &major, &minor, &revision);
 
-	success = TRUE;
+	if (!(cmajor > major
+	      || (cmajor == major && cminor > minor)
+	      || (cminor == minor && crevision > revision)))
+		goto check_old;
 
-	component_infos = e_component_registry_peek_list (shell->priv->component_registry);
-	for (p = component_infos; success && p != NULL; p = p->next) {
-		const EComponentInfo *info = p->data;
-		CORBA_Environment ev;
-		gboolean component_upgraded;
+	/* if upgrading from < 1.5, we need to copy most data from ~/evolution to ~/.evolution */
+	if (major == 1 && minor < 5) {
+		long size, space;
 
-		CORBA_exception_init (&ev);
-		
-		GNOME_Evolution_Component_upgradeFromVersion (info->iface, major, minor, revision, &ev);
-		
-		if (BONOBO_EX (&ev)) {
-			char *exception_text;
-			CORBA_char *id = CORBA_exception_id(&ev);
+		size = e_fsutils_usage(oldpath);
+		space = e_fsutils_avail(g_get_home_dir());
+		if (size != -1 && space != -1 && space < size) {
+			char *required = g_strdup_printf(_("%ld KB"), size);
+			char *have = g_strdup_printf(_("%ld KB"), space);
 
-			if (strcmp (id, ex_CORBA_NO_IMPLEMENT) == 0) {
-				/* Ignore components that do not implement this version, it
-				   might just mean that they don't need an upgrade path. */
-			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UpgradeFailed) == 0) {
-				GNOME_Evolution_Component_UpgradeFailed *ex = CORBA_exception_value(&ev);
-
-				res = e_error_run(NULL, "shell:upgrade-failed", ex->what, ex->why, NULL);
-				if (res == GTK_RESPONSE_CANCEL)
-					success = FALSE;
-			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UnsupportedVersion) == 0) {
-				/* This is non-fatal */
-				/* DO WE CARE??? */
-				printf("Upgrade of component failed, unsupported prior version\n");
-			} else {
-				exception_text = bonobo_exception_get_text (&ev);
-				res = e_error_run(NULL, "shell:upgrade-failed", exception_text, _("Uknown system error."), NULL);
-				g_free (exception_text);
-				if (res == GTK_RESPONSE_CANCEL)
-					success = FALSE;
-			}
+			e_error_run(NULL, "shell:upgrade-nospace", required, have, NULL);
+			g_free(required);
+			g_free(have);
+			_exit(0);
 		}
-		CORBA_exception_free (&ev);
 	}
 
-	return success;
-}
+	if (!attempt_upgrade (shell, major, minor, revision))
+		_exit(0);
 
+	/* mark as upgraded */
+	version_string = g_strdup_printf ("%s.%s", BASE_VERSION, UPGRADE_REVISION);
+	gconf_client_set_string (gconf_client, "/apps/evolution/version", version_string, NULL);
+	done_upgrade = TRUE;
+
+check_old:
+	/* if the last upgraded version was old, check for stuff to remove */
+	if (done_upgrade
+	    || 	(last_version = gconf_client_get_string (gconf_client, "/apps/evolution/last_version", NULL)) == NULL
+	    ||  sscanf(last_version, "%d.%d.%d", &lmajor, &lminor, &lrevision) != 3) {
+		lmajor = major;
+		lminor = minor;
+		lrevision = revision;
+	}
+	g_free(last_version);
+
+	if (lmajor == 1 && lminor < 5
+	    && stat(oldpath, &st) == 0
+	    && S_ISDIR(st.st_mode)) {
+		int res;
+
+		last_version = g_strdup_printf("%u.%u.%u", lmajor, lminor, lrevision);
+		res = e_error_run(NULL, "shell:upgrade-remove-1-4", last_version, NULL);
+		g_free(last_version);
+
+		switch (res) {
+		case GTK_RESPONSE_OK: /* 'delete' */
+			if (e_error_run(NULL, "shell:upgrade-remove-1-4-confirm", NULL) == GTK_RESPONSE_OK)
+				remove_dir(oldpath, oldpath);
+			else
+				break;
+			/* falls through */
+		case GTK_RESPONSE_ACCEPT: /* 'keep' */
+			lmajor = cmajor;
+			lminor = cminor;
+			lrevision = crevision;
+			break;
+		default:
+			/* cancel - noop */
+			break;
+		}
+	} else {
+		/* otherwise 'last version' is now the same as current */
+		lmajor = cmajor;
+		lminor = cminor;
+		lrevision = crevision;
+	}
+
+	last_version = g_strdup_printf("%u.%u.%u", lmajor, lminor, lrevision);
+	gconf_client_set_string (gconf_client, "/apps/evolution/last_version", last_version, NULL);
+	g_free(last_version);
+
+	g_free(oldpath);
+	g_object_unref (gconf_client);
+
+	return TRUE;
+}
 
 /**
  * e_shell_create_window:
@@ -779,10 +891,18 @@ e_shell_create_window (EShell *shell,
 
 	priv = shell->priv;
 
-	window = create_window (shell, component_id, template_window);
+	window = E_SHELL_WINDOW (e_shell_window_new (shell, component_id));
+
+	g_signal_connect (window, "delete_event", G_CALLBACK (window_delete_event_cb), shell);
+	g_object_weak_ref (G_OBJECT (window), window_weak_notify, shell);
+	shell->priv->windows = g_list_prepend (shell->priv->windows, window);
+
+	g_signal_emit (shell, signals[NEW_WINDOW_CREATED], 0, window);
 
 	gtk_widget_show (GTK_WIDGET (window));
 
+	e_error_default_parent((GtkWindow *)window);
+
 	set_interactive (shell, TRUE);
 
 	return window;
@@ -938,6 +1058,7 @@ offline_procedure_finished_cb (EShellOfflineHandler *offline_handler,
 {
 	EShell *shell;
 	EShellPrivate *priv;
+	ESEvent *ese;
 
 	shell = E_SHELL (data);
 	priv = shell->priv;
@@ -946,11 +1067,23 @@ offline_procedure_finished_cb (EShellOfflineHandler *offline_handler,
 		priv->line_status = E_SHELL_LINE_STATUS_OFFLINE;
 	else
 		priv->line_status = E_SHELL_LINE_STATUS_ONLINE;
+	e_passwords_set_online (!now_offline);
 
 	g_object_unref (priv->offline_handler);
 	priv->offline_handler = NULL;
 
 	g_signal_emit (shell, signals[LINE_STATUS_CHANGED], 0, priv->line_status);
+
+	/** @Event: Shell online state changed
+	 * @Id: state.changed
+	 * @Target: ESMenuTargetState
+	 * 
+	 * This event is emitted whenever the shell online state changes.
+	 *
+	 * Only the online and offline states are emitted.
+	 */
+	ese = es_event_peek();
+	e_event_emit((EEvent *)ese, "state.changed", (EEventTarget *)es_event_target_new_state(ese, TRUE));
 }
 
 /**
@@ -1000,7 +1133,8 @@ e_shell_go_online (EShell *shell,
 	EShellPrivate *priv;
 	GSList *component_infos;
 	GSList *p;
-	
+	ESEvent *ese;
+
 	g_return_if_fail (shell != NULL);
 	g_return_if_fail (E_IS_SHELL (shell));
 	g_return_if_fail (action_window == NULL || E_IS_SHELL_WINDOW (action_window));
@@ -1031,7 +1165,11 @@ e_shell_go_online (EShell *shell,
 	}
 
 	priv->line_status = E_SHELL_LINE_STATUS_ONLINE;
+	e_passwords_set_online (TRUE);
 	g_signal_emit (shell, signals[LINE_STATUS_CHANGED], 0, priv->line_status);
+
+	ese = es_event_peek();
+	e_event_emit((EEvent *)ese, "state.changed", (EEventTarget *)es_event_target_new_state(ese, TRUE));
 }
 
 
