@@ -40,7 +40,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include <gal/util/e-util.h>
 #include <gal/e-table/e-table-item.h>
 #include <gal/e-table/e-cell-text.h>
 
@@ -51,8 +50,6 @@
 /* This depends on ECalendar which is why I didn't put it in gal. */
 #include "e-calendar.h"
 
-static void e_cell_date_edit_class_init		(GtkObjectClass	*object_class);
-static void e_cell_date_edit_init		(ECellDateEdit	*ecde);
 static void e_cell_date_edit_destroy		(GtkObject	*object);
 static void e_cell_date_edit_get_arg		(GtkObject	*o,
 						 GtkArg		*arg,
@@ -115,18 +112,14 @@ enum {
 	ARG_UPPER_HOUR
 };
 
-static ECellPopupClass *parent_class;
-
-
-E_MAKE_TYPE (e_cell_date_edit, "ECellDateEdit", ECellDateEdit,
-	     e_cell_date_edit_class_init, e_cell_date_edit_init,
-	     e_cell_popup_get_type());
+G_DEFINE_TYPE (ECellDateEdit, e_cell_date_edit, E_CELL_POPUP_TYPE);
 
 
 static void
-e_cell_date_edit_class_init		(GtkObjectClass	*object_class)
+e_cell_date_edit_class_init (ECellDateEditClass *ecdec)
 {
-	ECellPopupClass *ecpc = (ECellPopupClass *) object_class;
+	GtkObjectClass	*object_class = (GtkObjectClass *) ecdec;
+	ECellPopupClass	*ecpc = (ECellPopupClass *) ecdec;
 
 	gtk_object_add_arg_type ("ECellDateEdit::show_time",
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE,
@@ -155,8 +148,6 @@ e_cell_date_edit_class_init		(GtkObjectClass	*object_class)
 	object_class->set_arg = e_cell_date_edit_set_arg;
 
 	ecpc->popup = e_cell_date_edit_do_popup;
-
-	parent_class = g_type_class_ref(e_cell_popup_get_type ());
 }
 
 
@@ -310,7 +301,7 @@ e_cell_date_edit_destroy		(GtkObject *object)
 	gtk_widget_destroy (ecde->popup_window);
 	ecde->popup_window = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (e_cell_date_edit_parent_class)->destroy (object);
 }
 
 
