@@ -29,8 +29,7 @@
 
 #include "e-config-page.h"
 
-#include <gtk/gtkwindow.h>
-
+#include <gtk/gtkdialog.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #ifdef __cplusplus
@@ -39,10 +38,10 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define E_TYPE_MULTI_CONFIG_DIALOG			(e_multi_config_dialog_get_type ())
-#define E_MULTI_CONFIG_DIALOG(obj)			(GTK_CHECK_CAST ((obj), E_TYPE_MULTI_CONFIG_DIALOG, EMultiConfigDialog))
-#define E_MULTI_CONFIG_DIALOG_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), E_TYPE_MULTI_CONFIG_DIALOG, EMultiConfigDialogClass))
-#define E_IS_MULTI_CONFIG_DIALOG(obj)			(GTK_CHECK_TYPE ((obj), E_TYPE_MULTI_CONFIG_DIALOG))
-#define E_IS_MULTI_CONFIG_DIALOG_CLASS(klass)		(GTK_CHECK_CLASS_TYPE ((obj), E_TYPE_MULTI_CONFIG_DIALOG))
+#define E_MULTI_CONFIG_DIALOG(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), E_TYPE_MULTI_CONFIG_DIALOG, EMultiConfigDialog))
+#define E_MULTI_CONFIG_DIALOG_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), E_TYPE_MULTI_CONFIG_DIALOG, EMultiConfigDialogClass))
+#define E_IS_MULTI_CONFIG_DIALOG(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), E_TYPE_MULTI_CONFIG_DIALOG))
+#define E_IS_MULTI_CONFIG_DIALOG_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((obj), E_TYPE_MULTI_CONFIG_DIALOG))
 
 
 typedef struct _EMultiConfigDialog        EMultiConfigDialog;
@@ -50,13 +49,13 @@ typedef struct _EMultiConfigDialogPrivate EMultiConfigDialogPrivate;
 typedef struct _EMultiConfigDialogClass   EMultiConfigDialogClass;
 
 struct _EMultiConfigDialog {
-	GnomeDialog parent;
+	GtkDialog parent;
 
 	EMultiConfigDialogPrivate *priv;
 };
 
 struct _EMultiConfigDialogClass {
-	GnomeDialogClass parent_class;
+	GtkDialogClass parent_class;
 };
 
 
